<?php
namespace SCM\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use Util\Validation;
use SCM\Traits\TResponse;
use Teknisa\Libs\Util\Utilities as LibUtilities;

class RequisicaoRetirada extends \Zeedhi\Framework\Controller\Simple {
    
    use TResponse;

	protected $requisicaoRetirada;
	protected $userService;
	protected $cardapioService;
	protected $environment;
	protected $pmeEndPoint;
	protected $efetivosService;
	protected $entityManager;
	protected $newCode;
	protected $libTecfood;

	public function __construct(\SCM\Services\RequisicaoRetirada $requisicaoRetirada, $userService, $cardapioService, $environment,
	    \Zeedhi\Framework\DependencyInjection\InstanceManager $instanceManager, $efetivosService, $entityManager, $newCode, $libTecfood,
	    LibUtilities $libUtilities) {
        $this->requisicaoRetirada = $requisicaoRetirada;
        $this->userService = $userService;
        $this->cardapioService = $cardapioService;
        $this->environment = $environment;
        $this->pmeEndPoint = $instanceManager->getParameter('pmeEndPoint').$instanceManager->getParameter('pmeBaseUri');
        $this->efetivosService = $efetivosService;
        $this->entityManager = $entityManager;
        $this->newCode = $newCode;
        $this->libTecfood = $libTecfood;
        $this->libUtilities = $libUtilities;
    }
    
    function dadosColetados(Request $request, Response $response) {
        try {
            $params = $request->getParameters();
            $productId = 7235;
            $unit = $params['unit'];
            $operador = $params['operador'];
            $nrorg = $this->environment->getNrorg();
    
            $units = explode('_', $unit);
    
            foreach ($units as $singleUnit) {
                $this->libUtilities->saveCollectedData($singleUnit, 'SCHOOLMEALSTEK', $productId, $operador, null, $nrorg);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    
    public function retornaInsumosPlanejados(Request $request, Response $response) {
        try {
            $params = $request->getParameters();
        	
        	$operador = $params['CDOPERADOR'];
        	$cdfilcentpme = $params['CDFILCENTPME'];
        	$cdfilial = $params['CDFILIAL'];
            $dtInicial = $params['DTINICIAL'];
            $dtFinal = $params['DTFINAL'];
            $nrorg = $this->environment->getNrorg();
            
            $tiposCardapio = $this->cardapioService->retornaTiposCardapio($cdfilcentpme, $dtInicial, $dtFinal, $nrorg);
            
            $servicosCardapio = $this->cardapioService->retornaServicosCardapio($cdfilcentpme, $cdfilial, $dtInicial, $dtFinal, $nrorg);
            
            $filiaisSelecionadas = $this->userService->retornaFiliaisUsuario($operador, $cdfilial);
            $centrais = [];
            foreach($filiaisSelecionadas['data'] as $unidade) { 
                $centrais[$unidade['CDFILCENTPME']][] = $unidade;
            }
            
            $errors = [];
            $consplans = array();
            $consplansConcat = "";
            /* PARA CADA CENTRAL CHAMA A EXECUÇÃO DA PROCEDURE CONSUMO_PLANEJADO_PME*/
            foreach($centrais as $central => $agregadas) {
                $agregadasConcat = implode('_', array_column($agregadas, 'CDFILIAL'));
                $tiposCardapioConcat = implode('_', array_column($tiposCardapio['data'], 'CDTIPOCARD'));
                $servicosConcat = implode('_', array_column($servicosCardapio['data'], 'CDSERVFILI'));
                // $result = $this->executeCallProcedureConsPlan($central, $agregadasConcat, $tiposCardapioConcat, $servicosConcat, $dtInicial, $dtFinal);
                $row = array(
                    'NRCONSPLAN' => null,
                    'GRUPFILI' => null,
                    'UNIDADE' => $agregadasConcat,
                    'GRUPOSERVICO' => null,
                    'SERVICO' => $servicosConcat,
                    'DTDE' => $dtInicial,
                    'DTATE' => $dtFinal,
                    'PRODINICIAL' => null,
                    'PRODFINAL' => null,
                    'CDFILCENTPME' => $central,
                    'TIPOCARD' => $tiposCardapioConcat,
                );
                $result = $this->callProcedure($row);
                $consplans[$central] = $result;
                if (!$result['error']) $consplansConcat .= $result['response']['result'][0]['NRCONSPLAN']."_";
                else $errors[] = $result;
            }
            if (!empty($errors)) $this->createLog($errors);
            $insumosPlanejados = $this->requisicaoRetirada->retornaInsumosPlanejados($cdfilcentpme, $dtInicial, $dtFinal, $consplansConcat);
            $this->sendResponse($insumosPlanejados);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    private function executeCallProcedureConsPlan($central, $agregadas, $tiposCardapio, $servicos, $dtInicial, $dtFinal) {
        $params = array(
            'requestType' => 'Row',
            'row'=> array(
                'NRCONSPLAN' => null,
                'GRUPFILI' => null,
                'UNIDADE' => $agregadas,
                'GRUPOSERVICO' => null,
                'SERVICO' => $servicos,
                'DTDE' => $dtInicial,
                'DTATE' => $dtFinal,
                'PRODINICIAL' => null,
                'PRODFINAL' => null,
                'CDFILCENTPME' => $central,
                'TIPOCARD' => $tiposCardapio,
            )
        );
        
        $curl = curl_init();
        
        $hash = $_SERVER["HTTP_OAUTH_HASH"] ? $_SERVER["HTTP_OAUTH_HASH"] : '';
        $project = $_SERVER["HTTP_OAUTH_PROJECT"] ? $_SERVER["HTTP_OAUTH_PROJECT"] : '';
        $token = $_SERVER["HTTP_OAUTH_TOKEN"] ? $_SERVER["HTTP_OAUTH_TOKEN"] : '';
        
        curl_setopt_array($curl, array(
          CURLOPT_URL => str_replace('//backend', '/backend', $this->pmeEndPoint).'/callProcedureConsPlanPME',
          CURLOPT_RETURNTRANSFER => true,
          CURLOPT_ENCODING => '',
          CURLOPT_MAXREDIRS => 10,
          CURLOPT_TIMEOUT => 0,
          CURLOPT_FOLLOWLOCATION => true,
          CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
          CURLOPT_CUSTOMREQUEST => 'POST',
          CURLOPT_POSTFIELDS => json_encode($params),
          CURLOPT_HTTPHEADER => array(
            'Connection: keep-alive',
            'Content-Type: application/json',
            // 'Cookie: PHPSESSID=s0jkdm12n3hftkahq1q7387tv0',
            'Language: pt-BR',
            // 'OAuth-Hash: '. $hash,
            // 'OAuth-KeepConnected: null',
            // 'OAuth-Project:' . $project,
            // 'OAuth-Token: ' . $token,
          ),
        ));
        
        $response = curl_exec($curl);
        $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        $err    = curl_errno($curl);
        $errmsg = curl_error($curl);
        
        curl_close($curl);
        
        if($err || empty($response) || $httpCode !== 200){
            // throw new \Exception("ERRO NA CHAMADA DA PROCEDURE: ". $err. $errmsg. " \n");
            return array('error' => true, 'message' => "ERRO NA CHAMADA DA PROCEDURE: ".$errmsg, 'response' => $response);
        }
        
        $resultProcedure = json_decode($response, true);
        
        if (!\Zeedhi\Framework\Util\Functions::arrayKeyExists('dataset', $resultProcedure)) {
            // throw new \Exception("ERRO NA CHAMADA DA PROCEDURE: ". $errmsg. " \n");
            return array('error' => true, 'message' => "ERRO NO CONTEUDO DA RESPOSTA DA PROCEDURE: ".$errmsg, 'resultProcedure' => $resultProcedure, 'response' => $response);
        }
        return array('error' => false, 'message' => 'success', 'response' => $resultProcedure['dataset']);
    }
    
    private function createLog($errors) {
        $folder = "../log";
        $fileName = "insumosPlanejadosError-".round(microtime(true) * 1000).".json";
        $currentFile = $folder . "/" . $fileName;
        if (file_exists($currentFile) === false) {
            if(!file_exists($folder)){
                mkdir($folder, 0775, true);
            }
            $content = json_encode($errors, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE);
            file_put_contents($currentFile, $content);
        }
    }
    
    public function salvarRequisicao(Request $request, Response $response) {
        $this->entityManager->getConnection()->beginTransaction();
        try {
            $rows = $request->getDataSet()->getRows();
            $row = $rows[0];
            $nrorg = $this->environment->getNrorg();
            $cdoperador = $this->environment->getCdOperador();
            $efetivos = $this->efetivosService->retornaEfetivosRequisicao($nrorg, $row['CDFILICPLA'], $row['DTCONSPLAN'],
                $row['DTCONSPLAN'], $row['CDSERVCPLA'], $row['CDTPCARDPME']);
            $updatedEfetivos = false;
            if ((\Zeedhi\Framework\Util\Functions::arrayKeyExists('data', $efetivos) && empty($efetivos['data']))) {
                $cardiapme = $this->requisicaoRetirada->getCardiapme($row['CDFILCENTPME'], $row['CDSERVCPLA'], $row['CDFILICPLA'], $row['DTCONSPLAN']);
                if (!empty($cardiapme)) {
                    $this->requisicaoRetirada->updateEfetivoDia($row['CDFILCENTPME'], $cardiapme['NRCARDPME'], $cardiapme['NRCARDIAPME'],
                        $row['CDSERVCPLA'], $row['CDFILICPLA'], $row['DTCONSPLAN'], $row['QTCOMENSAIS'], $row['QTCOMENSAIS']);
                    $updatedEfetivos = true;
                }
            } else if ((\Zeedhi\Framework\Util\Functions::arrayKeyExists('data', $efetivos) && \Zeedhi\Framework\Util\Functions::arrayKeyExists('QTCOMEFECPME', $efetivos['data'][0])
                    && $efetivos['data'][0]['QTCOMEFECPME'] == 0)) {
                 $this->requisicaoRetirada->updateEfetivoDia($row['CDFILCENTPME'], $efetivos['data'][0]['NRCARDPME'], $efetivos['data'][0]['NRCARDIAPME'],
                    $row['CDSERVCPLA'], $row['CDFILICPLA'], $row['DTCONSPLAN'], $row['QTCOMENSAIS'], $row['QTCOMENSAIS']);   
                $updatedEfetivos = true;
            }
            $this->entityManager->getConnection()->commit();
           //  $response->addDataSet(new DataSet ('updatedEfetivos', [$updatedEfetivos]));
        } catch (\Exception $e) {
            $this->entityManager->getConnection()->rollback();
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    // public function callProcedure(DTO\Request\Row $request, DTO\Response $response) {
    public function callProcedure($rows) {
    //   set_time_limit(300);
      ini_set('memory_limit', '3072M');
      // if (empty($_SERVER["HTTP_OAUTH_TOKEN"])) throw new \Exception("Acesso negado.");
    //   $rows = $request->getRow(); 
      $nrorg = $this->environment->getNrOrg();
      $rows['NRCONSPLAN'] = empty($rows['NRCONSPLAN']) ? $this->newCode->getCodeByTableName('CONSPLAN'.$rows['CDFILCENTPME'], $nrorg) : $rows['NRCONSPLAN'];
      $result = $this->requisicaoRetirada->procedure($rows);
      return $result;
    //   $response->addDataSet(new DataSet("result", array($result)));
    }
    
    public function salvarRetirada(Request $request, Response $response) {
        // $this->entityManager->getConnection()->beginTransaction();
        try {
            $rows = $request->getDataSet()->getRows();
            $row = (array) $rows[0];
            $retorno = $this->libTecfood->incluirRetirada($row);
            if ((\Zeedhi\Framework\Util\Functions::arrayKeyExists('mensagens', $retorno) && !empty($retorno['mensagens'])) && count(array_filter($retorno['mensagens'], function($a) { return $a['msg'] !== 'retirada'; })) > 0) {
                $this->sendResponse(array('error' => array_filter($retorno['mensagens'], function($a) { return $a['msg'] !== 'retirada'; })));
            }
            // $this->entityManager->getConnection()->commit();
        } catch (\Exception $e) {
            // $this->entityManager->getConnection()->rollback();
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
}