<?php
/**
 * Created by Luiz Filho
 */

namespace SCM\Model\Repository;

use Doctrine\ORM\EntityRepository;
use SCM\Traits\TQuery;

class Complemento extends EntityRepository {
    
    use TQuery;

    public function retornaProdutos($nrorg, $filiais, $cdgrupprod, $qtdInicial = null, $qtdFinal = null) {
        $params = array(
            'CDFILIAL' => $filiais,
        );
        
        if (!empty($qtdInicial)) $params['ROWNUMINI'] = $qtdInicial;
        if (!empty($qtdFinal)) $params['ROWNUMFIM'] = $qtdFinal;

        if ($cdgrupprod == 'T') {
            $sql = " SELECT PRODUTO.NMPRODUTO, PRODUTO.CDPRODUTO, PRODUTO.CDARVPROD, PRODUTO.SGUNIDADE,
                        DECODE(PRODUTO.CDPRODUTO, PRODUTO.CDPRODESTO, 'E', 'C') AS TIPOPRODUTO, PRODFILI.CDFILIAL,
                        NVL(PRODFILI.IDCNTRESTOQ, IDCNTRESTOQ) AS IDCNTRESTOQ
                        FROM PRODFILI
                        JOIN PRODUTO ON PRODFILI.CDPRODUTO = PRODUTO.CDPRODUTO 
                        WHERE INSTR(:CDFILIAL, CDFILIAL) <> 0
                        AND PRODFILI.IDPRODATIV = 'A'
                        AND PRODUTO.CDNVPRODUTO = (SELECT CDNVPRODUTO FROM ( SELECT (CDNVPRODUTO) FROM PRODUTO ORDER BY 1 DESC) WHERE ROWNUM = 1)
                        AND PRODUTO.IDPRODATIVPR = 'S'
                        ORDER BY CDARVPROD, NMPRODUTO
                        ";
        } else {
            $sql = "SELECT PRODUTO.NMPRODUTO, PRODUTO.CDPRODUTO, PRODUTO.CDARVPROD, PRODUTO.SGUNIDADE, DECODE(PRODUTO.CDPRODUTO, PRODUTO.CDPRODESTO, 'E', 'C') AS TIPOPRODUTO, PRODFILI.CDFILIAL,
                    NVL(PRODFILI.IDCNTRESTOQ, IDCNTRESTOQ) AS IDCNTRESTOQ
                    FROM PRODFILI
                    JOIN PRODUTO ON PRODFILI.CDPRODUTO = PRODUTO.CDPRODUTO 
                    JOIN PRSUBGRPROD ON PRODUTO.CDPRODUTO = PRSUBGRPROD.CDPRODUTO 
                    WHERE PRODUTO.IDPRODATIVPR = 'S'
                    AND PRODUTO.CDNVPRODUTO = (SELECT CDNVPRODUTO FROM ( SELECT (CDNVPRODUTO) FROM PRODUTO ORDER BY 1 DESC) WHERE ROWNUM = 1)
                    AND (PRSUBGRPROD.CDGRUPPROD = :CDGRUPPROD)
                    AND PRSUBGRPROD.CDFILISUBGR = 'P'
                    AND PRODFILI.IDPRODATIV = 'A'
                    AND INSTR(:CDFILIAL, PRODFILI.CDFILIAL) <> 0
                    ORDER BY CDARVPROD, NMPRODUTO";
            $params['CDGRUPPROD'] = $cdgrupprod;
        }
        return $this->createAndExecuteQueryAndBuildResponseData($this->getEntityManager(), $sql, $params);
    }

    public function retornaJustificativas($nrorg) {

        $sql = "SELECT
            CDJUSTSOL, DSJUSTSOL AS DSJUSTOPER, DSJUSTSOL AS DSJUSTSOLEXT
          FROM JUSTSOLI
          ORDER BY DSJUSTSOL";

        $params = array();
        
        return $this->createAndExecuteQueryAndBuildResponseData($this->getEntityManager(), $sql, $params);
    }

    public function retornaProdutosInventarios($nrorg, $filiais) {

        $sql = "SELECT PF.CDFILIAL, PF.CDPRODUTO, P.NMPRODUTO, P.CDARVPROD, SGUNIDADE
            FROM PRODFILI PF JOIN PRODUTO P ON PF.CDPRODUTO = P.CDPRODUTO 
            WHERE CDFILIAL IN ($filiais)
            AND IDPRODATIV  = 'A'
            AND IDPRODATIVPR = 'S'
            AND IDCNTRESTOQ = 'S'
            AND P.CDPRODUTO = CDPRODESTO
            ORDER BY P.NMPRODUTO ASC";

        $params = array();

        $query = $this->getEntityManager()->getConnection()->prepare($sql);
        $query->execute($params);
        $result = $query->fetchAll();
        return $result;
    }

    public function retornaGruposProduto($nrorg){

        $sql = "SELECT CDGRUPPROD, NMGRUPPROD FROM GRUPPROD ORDER BY CDGRUPPROD";

        $params = array();

        return $this->createAndExecuteQueryAndBuildResponseData($this->getEntityManager(), $sql, $params);
    }

    public function retornaMascaraProduto($nrorg){
        $sql = "SELECT CDPICTPROD FROM PARAMGERAL";

        $params = array();

        $query = $this->getEntityManager()->getConnection()->prepare($sql);
        $query->execute($params);
        $result = $query->fetchAll();
        return $result;

    }

    public function retornaUsuarioAtivo($dsemailoper, $nrorg) {
        $sql = "SELECT IDOPERATIVO FROM OPERADOR WHERE DSEMAILOPER = :DSEMAILOPER";

        $params = array(
            'DSEMAILOPER' => $dsemailoper,
        );

        $query = $this->getEntityManager()->getConnection()->prepare($sql);
        $query->execute($params);
        $result = $query->fetchAll();
        return $result;
    }
    
    public function retornaOperadorAtivo($cdoperador, $nrorg) {
        $sql = "SELECT IDOPERATIVO FROM OPERADOR WHERE CDOPERADOR = :CDOPERADOR";

        $params = array(
            'CDOPERADOR' => $cdoperador,
        );

        $query = $this->getEntityManager()->getConnection()->prepare($sql);
        $query->execute($params);
        $result = $query->fetchAll();
        return $result;
    }

    public function retornaQtdTotalProdutos($nrorg, $filiais, $cdgrupprod) {
        $params = array(
            'CDFILIAL' => $filiais,
        );

        if ($cdgrupprod == 'T') {

            $sql = "SELECT COUNT(*) AS QTD
                    FROM (
                        SELECT PRODUTO.NMPRODUTO, PRODUTO.CDPRODUTO, PRODUTO.CDARVPROD, PRODUTO.SGUNIDADE,
                        DECODE(PRODUTO.CDPRODUTO, PRODUTO.CDPRODESTO, 'E', 'C') AS TIPOPRODUTO, PRODFILI.CDFILIAL
                        FROM PRODFILI
                        JOIN PRODUTO ON PRODFILI.CDPRODUTO = PRODUTO.CDPRODUTO 
                        WHERE CDFILIAL = :CDFILIAL
                        AND PRODFILI.IDPRODATIV = 'A'
                        AND PRODUTO.CDNVPRODUTO = (SELECT MAX(CDNVPRODUTO) FROM PRODUTO)
                        AND PRODUTO.IDPRODATIVPR = 'S'
                    )";
        } else {

            $sql = "SELECT COUNT(*) AS QTD
                    FROM (
                        SELECT PRODUTO.NMPRODUTO, PRODUTO.CDPRODUTO, PRODUTO.CDARVPROD, PRODUTO.SGUNIDADE, PRODUTO.CDPRODESTO, PRODFILI.CDFILIAL
                        FROM PRODUTO
                        JOIN PRODFILI ON PRODUTO.CDPRODUTO = PRODFILI.CDPRODUTO 
                        JOIN PRSUBGRPROD ON PRODUTO.CDPRODUTO = PRSUBGRPROD.CDPRODUTO 
                        WHERE PRODUTO.IDPRODATIVPR = 'S'
                        AND PRODUTO.CDNVPRODUTO = (SELECT MAX(CDNVPRODUTO) FROM PRODUTO)
                        AND (PRSUBGRPROD.CDGRUPPROD = :CDGRUPPROD)
                        AND PRSUBGRPROD.CDFILISUBGR = 'P'
                        AND PRODFILI.IDPRODATIV = 'A'
                        AND PRODFILI.CDFILIAL = :CDFILIAL
                        GROUP BY PRODUTO.NMPRODUTO, PRODUTO.CDPRODUTO, PRODUTO.CDARVPROD, PRODUTO.SGUNIDADE, PRODUTO.CDPRODESTO, PRODFILI.CDFILIAL
                    )";

            $params['CDGRUPPROD'] = $cdgrupprod;
        }

        $query = $this->getEntityManager()->getConnection()->prepare($sql);
        $query->execute($params);
        $result = $query->fetchAll();
        return $result;
    }
}