<?php
/**
 * Created by Luiz Filho
 */

namespace SCM\Model\Repository;

use Doctrine\ORM\EntityRepository;
use SCM\Traits\TQuery;

class Efetivos extends EntityRepository {

    use TQuery;

    private function getDataSet($sql, $params, $filiais) {
        $dataSet = array();
        foreach ($filiais as $filial) {
            $params['CDFILIAL'] = $filial;
            $query = $this->getEntityManager()->getConnection()->prepare($sql);
            $query->execute($params);
            $dataSet = array_merge($dataSet, $query->fetchAll());
        }
        return $dataSet;
    }

    public function retornaDatasCardapio($nrorg, $filiais, $dataInicial, $dataFinal) {
	  	$sql = "SELECT DISTINCT TO_CHAR(DTCARDIAPME, 'DD/MM/YYYY') AS DATA,
            TO_CHAR(DTCARDIAPME, 'DD/MM/YYYY - DAY', 'NLS_DATE_LANGUAGE= ''BRAZILIAN PORTUGUESE''') AS DATA_EXTENSO,
            CDFILIAL
            FROM CARDIAPME
            WHERE CDFILIAL = :CDFILIAL
            AND DTCARDIAPME BETWEEN TO_DATE(:DATA_INICIAL, 'DD/MM/YYYY') AND TO_DATE(:DATA_FINAL, 'DD/MM/YYYY')
            ORDER BY TO_DATE(DATA, 'DD/MM/YYYY') ASC";

	  	$params = array(
            'DATA_INICIAL' => $dataInicial,
            'DATA_FINAL' => $dataFinal
		);

        return $this->getDataSet($sql, $params, $filiais);
    }

    public function retornaTiposCardapio($nrorg, $filiais, $dataInicial, $dataFinal) {
        $sql = "SELECT CD.CDTIPOCARD, TC.NMTIPOCARD, CA.CDFILIAL
                FROM CARDIAPME CA
                JOIN SERVICO SE ON  SE.CDFILIAL = CA.CDFILIAL
                    AND SE.CDSERVFILI = CA.CDSERVFILI
                JOIN CARDAPIOPME CD ON  CD.CDFILCENTPME = CA.CDFILCENTPME
                    AND CD.NRCARDPME = CA.NRCARDPME
                JOIN SERVTIPOCARD SC ON  SC.CDSERVFILI = CA.CDSERVFILI
                    AND SC.CDFILCENTPME = CA.CDFILCENTPME
                    AND SC.CDTIPOCARD = CD.CDTIPOCARD
                JOIN TIPOCARD TC ON  TC.CDTIPOCARD = CD.CDTIPOCARD
                    AND TC.CDFILCENTPME = CA.CDFILCENTPME
                WHERE CA.CDFILIAL = :CDFILIAL
                    AND CA.DTCARDIAPME BETWEEN TO_DATE(:DATA_INICIAL, 'DD/MM/YYYY') AND TO_DATE(:DATA_FINAL, 'DD/MM/YYYY')
                GROUP BY CD.CDTIPOCARD, TC.NMTIPOCARD, CA.CDFILIAL
                ORDER BY  CD.CDTIPOCARD ASC";

        $params = array(
            'DATA_INICIAL' => $dataInicial,
            'DATA_FINAL' => $dataFinal
        );

        return $this->getDataSet($sql, $params, $filiais);
    }

    public function retornaCardapio($nrorg, $filiais, $dataInicial, $dataFinal) {
        $sql = "SELECT CD.DSCARDPME, CA.NRCARDPME, CA.CDFILIAL
                FROM CARDIAPME CA
                JOIN SERVICO SE ON  SE.CDFILIAL = CA.CDFILIAL
                    AND SE.CDSERVFILI = CA.CDSERVFILI
                JOIN CARDAPIOPME CD ON  CD.CDFILCENTPME = CA.CDFILCENTPME
                    AND CD.NRCARDPME = CA.NRCARDPME
                JOIN SERVTIPOCARD SC ON  SC.CDSERVFILI = CA.CDSERVFILI
                    AND SC.CDFILCENTPME = CA.CDFILCENTPME
                    AND SC.CDTIPOCARD = CD.CDTIPOCARD
                JOIN TIPOCARD TC ON  TC.CDTIPOCARD = CD.CDTIPOCARD
                    AND TC.CDFILCENTPME = CA.CDFILCENTPME
                WHERE CA.CDFILIAL = :CDFILIAL
                    AND CA.DTCARDIAPME BETWEEN TO_DATE(:DATA_INICIAL, 'DD/MM/YYYY') AND TO_DATE(:DATA_FINAL, 'DD/MM/YYYY')
                GROUP BY CD.DSCARDPME, CA.CDFILCENTPME, CA.NRCARDPME, CA.CDFILIAL
                ORDER BY  CA.NRCARDPME ASC";

        $params = array(
            'DATA_INICIAL' => $dataInicial,
            'DATA_FINAL' => $dataFinal
        );

        return $this->getDataSet($sql, $params, $filiais);
    }

    public function retornaServicosCardapio($nrorg, $filial, $dataInicial, $dataFinal) {
        $sql = "SELECT ROWNUM, TC.CDTIPOCARD, TC.NMTIPOCARD, CD.DSCARDPME, CA.CDFILCENTPME,
                    CA.NRCARDPME, CA.NRCARDIAPME, CA.CDSERVFILI, CA.CDFILIAL,
                    TO_CHAR(CA.DTCARDIAPME, 'DD/MM/YYYY - DAY', 'NLS_DATE_LANGUAGE= ''BRAZILIAN PORTUGUESE''') AS DATA_EXTENSO,
                    TO_CHAR(CA.DTCARDIAPME, 'DD/MM/YYYY') AS DTCARDIAPME, CA.IDCARDESPEC,
                    SE.NMSERVICO, QTCOMEFECPME, QTCLIEFECPME, QTFUNEFECPME, QTDOAEFECPME, 
                    CA.CDFILCENTPME || CA.NRCARDPME || CA.NRCARDIAPME || CA.CDFILIAL || CA.CDSERVFILI || TC.CDTIPOCARD AS KEY,
                    TO_CHAR(CA.DTCARDIAPME, 'DD/MM/YYYY') AS DTINICIAL, TO_CHAR(CA.DTCARDIAPME, 'DD/MM/YYYY') AS DTFINAL
                FROM CARDIAPME CA
                JOIN SERVICO SE ON SE.CDFILIAL = CA.CDFILIAL
                    AND SE.CDSERVFILI = CA.CDSERVFILI
                JOIN CARDAPIOPME CD ON CD.CDFILCENTPME = CA.CDFILCENTPME
                    AND CD.NRCARDPME = CA.NRCARDPME
                JOIN SERVTIPOCARD SC ON SC.CDSERVFILI = CA.CDSERVFILI
                    AND SC.CDFILCENTPME = CA.CDFILCENTPME
                    AND SC.CDTIPOCARD = CD.CDTIPOCARD
                JOIN TIPOCARD TC ON TC.CDTIPOCARD = SC.CDTIPOCARD
                    AND TC.CDFILCENTPME = CA.CDFILCENTPME
                WHERE INSTR(:CDFILIAL, CA.CDFILIAL) <> 0 
                    AND CA.DTCARDIAPME BETWEEN TO_DATE(:DATA_INICIAL, 'DD/MM/YYYY') AND TO_DATE(:DATA_FINAL, 'DD/MM/YYYY')        
                ORDER BY TO_DATE(DTCARDIAPME, 'DD/MM/YYYY'), CA.CDSERVFILI ASC";
            // CA.CDFILIAL = :CDFILIAL
        $params = array(
            'DATA_INICIAL' => $dataInicial,
            'DATA_FINAL' => $dataFinal,
            'CDFILIAL' => $filial,
        );

        // return $this->getDataSet($sql, $params, $filiais);
        return $this->createAndExecuteQueryAndBuildResponseData($this->getEntityManager(), $sql, $params);
    }

    public function retornaClientesDoServico($nrorg, $filiais, $dataInicial, $dataFinal, $cdservfili, $cdtipocard) {
        $sql = "SELECT ROWNUM, S.CDFILCENTPME, S.CDTIPOCARD, S.CDFILIAL, S.CDSERVFILI, S.CDCLIENTE, C.NMRAZSOCCLIE,
                NVL(TO_CHAR(E.DTEFETCLPME, 'DD/MM/YYYY'), TO_CHAR(D.DTDATA, 'DD/MM/YYYY')) AS DTEFETCLPME,
                E.QTCOMEFCLPME, E.QTFUNEFCLPME, E.QTDOAEFCLPME, 
                (E.QTCOMEFCLPME + E.QTFUNEFCLPME + E.QTDOAEFCLPME) AS TOTAL, NVL(IDCLIENPRIN, 'N') IDCLIENPRIN,
                NVL(TO_CHAR(E.DTEFETCLPME, 'DD/MM/YYYY'), TO_CHAR(D.DTDATA, 'DD/MM/YYYY')) AS DTINICIAL,
                NVL(TO_CHAR(E.DTEFETCLPME, 'DD/MM/YYYY'), TO_CHAR(D.DTDATA, 'DD/MM/YYYY')) AS DTFINAL,
                S.CDFILCENTPME || S.CDTIPOCARD || S.CDFILIAL || S.CDSERVFILI || S.CDCLIENTE || TO_CHAR(E.DTEFETCLPME, 'DDMMYYYY') AS KEY,
                E.NRAPURACFAT
            FROM SERVCLIEPME S
            INNER JOIN CLIENTE C ON C.CDCLIENTE = S.CDCLIENTE
            CROSS JOIN DATA D
            LEFT JOIN EFETCLIEPME E ON E.CDFILIAL = S.CDFILIAL AND
                E.CDSERVFILI = S.CDSERVFILI AND
                E.CDCLIENTE = S.CDCLIENTE AND
                E.DTEFETCLPME = D.DTDATA 
            LEFT JOIN CLIENFILIAL CF ON
                S.CDCLIENTE = CF.CDCLIENTE AND
                CF.CDFILIAL = S.CDFILCENTPME 
            WHERE INSTR(:CDFILIAL, S.CDFILIAL) <> 0 
                AND ( INSTR(:CDSERVFILI, S.CDSERVFILI) <> 0 OR ( :CDSERVFILI = 'T' ) )
                AND ( INSTR(:CDTIPOCARD, S.CDTIPOCARD) <> 0 OR ( :CDTIPOCARD = 'T' ) )
                AND D.DTDATA BETWEEN TO_DATE(:DATA_INICIAL, 'DD/MM/YYYY') AND TO_DATE(:DATA_FINAL, 'DD/MM/YYYY')
            ORDER BY C.NMRAZSOCCLIE, D.DTDATA";
// S.CDFILIAL = :CDFILIAL
// AND S.CDTIPOCARD = :CDTIPOCARD
// AND S.CDSERVFILI = :CDSERVFILI
        $params = array(
            'DATA_INICIAL' => $dataInicial,
            'DATA_FINAL' => $dataFinal,
            'CDFILIAL' => $filiais,
            'CDSERVFILI' => $cdservfili,
            'CDTIPOCARD' => $cdtipocard,
        );

        // return $this->getDataSet($sql, $params, $filiais);
        return $this->createAndExecuteQueryAndBuildResponseData($this->getEntityManager(), $sql, $params);
    }
    
    public function retornaEfetivosRequisicao($nrorg, $filial, $dataInicial, $dataFinal, $cdservfili, $cdtipocard) {
        $sql = "SELECT CA.NRCARDPME, NRCARDIAPME, QTCOMEFECPME, QTCLIEFECPME, QTFUNEFECPME, QTDOAEFECPME
                FROM CARDIAPME CA
                JOIN SERVICO SE ON SE.CDFILIAL = CA.CDFILIAL
                    AND SE.CDSERVFILI = CA.CDSERVFILI
                JOIN CARDAPIOPME CD ON CD.CDFILCENTPME = CA.CDFILCENTPME
                    AND CD.NRCARDPME = CA.NRCARDPME
                JOIN SERVTIPOCARD SC ON SC.CDSERVFILI = CA.CDSERVFILI
                    AND SC.CDFILCENTPME = CA.CDFILCENTPME
                    AND SC.CDTIPOCARD = CD.CDTIPOCARD
                JOIN TIPOCARD TC ON TC.CDTIPOCARD = SC.CDTIPOCARD
                    AND TC.CDFILCENTPME = CA.CDFILCENTPME
                WHERE INSTR(:CDFILIAL, CA.CDFILIAL) <> 0 
                    AND ( INSTR(:CDSERVFILI, SC.CDSERVFILI) <> 0 OR ( :CDSERVFILI = 'T' ) )
                    AND ( INSTR(:CDTIPOCARD, SC.CDTIPOCARD) <> 0 OR ( :CDTIPOCARD = 'T' ) )
                    AND CA.DTCARDIAPME BETWEEN TO_DATE(:DATA_INICIAL, 'DD/MM/YYYY') AND TO_DATE(:DATA_FINAL, 'DD/MM/YYYY')   ";
            // CA.CDFILIAL = :CDFILIAL
        $params = array(
            'DATA_INICIAL' => $dataInicial,
            'DATA_FINAL' => $dataFinal,
            'CDFILIAL' => $filial,
            'CDSERVFILI' => $cdservfili,
            'CDTIPOCARD' => $cdtipocard,
        );
        
        return $this->createAndExecuteQueryAndBuildResponseData($this->getEntityManager(), $sql, $params);
    }
}