<?php
/**
 * Created by Luiz Filho
 */

namespace SCM\Model\Repository;

use Doctrine\ORM\EntityRepository;
use SCM\Traits\TQuery;

class Inventarios extends EntityRepository {
    
    use TQuery;

    public function retornaUltimoInventario($nrorg, $filiais) {
        $dataSet = array();
	  	$sql = "SELECT I.CDFILIAL, I.NRINVENTRETI, NRITINVENTRETI, I.CDPRODUTO, P.NMPRODUTO, P.CDARVPROD,
	  	    I.CDFILIAL || I.NRINVENTRETI || I.CDPRODUTO AS KEY
            FROM ITEMINVENTRETI I, PRODUTO P,
            (SELECT MAX(NRINVENTRETI) NRINVENTRETI, CDFILIAL
                    FROM INVENTARIORETI
                    WHERE INSTR(:CDFILIAL, CDFILIAL) <> 0 
                    AND LENGTH(NRINVENTRETI) = 10
                    GROUP BY CDFILIAL
                ) NR
            WHERE I.NRINVENTRETI = NR.NRINVENTRETI
            AND I.CDFILIAL  = NR.CDFILIAL
            AND P.CDPRODUTO = I.CDPRODUTO
            ORDER BY P.NMPRODUTO ASC";

	  	$params = array(
		    'CDFILIAL' => $filiais,
		);
        // foreach ($filiais as $filial) {
        //     $params['CDFILIAL'] = $filial;
        //     $query = $this->getEntityManager()->getConnection()->prepare($sql);
        //     $query->execute($params);
        //     $dataSet = array_merge($dataSet, $query->fetchAll());
        // }
        // return $dataSet;
        return $this->createAndExecuteQueryAndBuildResponseData($this->getEntityManager(), $sql, $params);
    }

    public function retornaProximoCodigoInventario($nrorg, $filiais) {
        $dataSet = array();
        $sql = "SELECT LPAD((NVL(MAX(NRINVENTRETI),0) + 1),10,'0') PROXIMOCODIGO,
            CDFILIAL
            FROM INVENTARIORETI
            WHERE INSTR(:CDFILIAL, CDFILIAL) <> 0 
            AND LENGTH(NRINVENTRETI) = 10 
            GROUP BY CDFILIAL";

        $params = array(
            'CDFILIAL' => $filiais,
        );
        // foreach ($filiais as $filial) {
        //     $params['CDFILIAL'] = $filial;
        //     $query = $this->getEntityManager()->getConnection()->prepare($sql);
        //     $query->execute($params);
        //     $dataSet = array_merge($dataSet, $query->fetchAll());
        // }
        // return $dataSet;
        return $this->createAndExecuteQueryAndBuildResponseData($this->getEntityManager(), $sql, $params);
    }

    public function retornaProximoCodigoProdutoInventario($nrorg, $filial, $nrinvent) {
        $dataSet = array();
        $sql = "SELECT LPAD((NVL(MAX(NRITINVENTRETI),0) + 1),4,'0') PROXIMOCODIGO,
            CDFILIAL, NRINVENTRETI
            FROM ITEMINVENTRETI
            WHERE CDFILIAL = :CDFILIAL
            AND NRINVENTRETI = :NRINVENTRETI
            AND LENGTH(NRITINVENTRETI) = 4 
            GROUP BY CDFILIAL, NRINVENTRETI";

        $params = array(
            'CDFILIAL' => $filial,
            'NRINVENTRETI' => $nrinvent,
        );
        // foreach ($filiais as $filial) {
        //     $params['CDFILIAL'] = $filial;
        //     $query = $this->getEntityManager()->getConnection()->prepare($sql);
        //     $query->execute($params);
        //     $dataSet = array_merge($dataSet, $query->fetchAll());
        // }
        $query = $this->getEntityManager()->getConnection()->prepare($sql);
        $query->execute($params);
        return $query->fetchAll();
    }

    public function retornaDatasInventarios($nrorg, $filiais, $dtinicio, $dtfinal) {
        $dataSet = array();
        $sql = "SELECT CDFILIAL || NRINVENTRETI AS KEY,
                NRINVENTRETI,
              DSINVENTRETI,
              CDFILIAL,
              TO_CHAR(DTINVENTRETI, 'DD/MM/YYYY') AS DTINVENTRETI,
              TO_CHAR(DTINVENTRETI, 'DD/MM/YYYY') AS DTINICIAL,
              TO_CHAR(DTINVENTRETI, 'DD/MM/YYYY') AS DTFINAL
            FROM INVENTARIORETI
            WHERE INSTR(:CDFILIAL, CDFILIAL) <> 0 
            AND IDRETIEFET = 'N'
            AND DTINVENTRETI BETWEEN TO_DATE(:DATA_INICIAL, 'DD/MM/YYYY') AND TO_DATE(:DATA_FINAL, 'DD/MM/YYYY')
            ORDER BY TO_DATE(DTINVENTRETI, 'DD/MM/YYYY') DESC";

        $params = array(
            'CDFILIAL' => '',
            'DATA_FINAL' => $dtfinal,
            'DATA_INICIAL' => $dtinicio
        );
        
        if (!is_array($filiais)) $filiais = [$filiais];
        $params['CDFILIAL'] = implode("_", $filiais);
        // foreach ($filiais as $filial) {
            // $params['CDFILIAL'] = $filial;
        //     $query = $this->getEntityManager()->getConnection()->prepare($sql);
        //     $query->execute($params);
        //     $dataSet = array_merge($dataSet, $query->fetchAll());
        // }
        // return $dataSet;
        
        return $this->createAndExecuteQueryAndBuildResponseData($this->getEntityManager(), $sql, $params);
    }

    public function retornaProdutosInventarios($nrorg, $cdfilial, $nrinvent) {

        $sql = "SELECT ROWNUM, CDFILIAL, NRINVENTRETI, NRITINVENTRETI, QTITINVENTRETI, I.CDPRODUTO,
            P.NMPRODUTO, P.CDARVPROD, I.CDOPERINCLUSAO, I.DTINCLUSAO, I.CDOPERULTATU, I.DTULTATU, 
            CDFILIAL || NRINVENTRETI || NRITINVENTRETI AS KEY
            FROM ITEMINVENTRETI I, PRODUTO P
            WHERE P.CDPRODUTO = I.CDPRODUTO
            AND INSTR (:CDFILIAL, I.CDFILIAL) <> 0
            AND I.NRINVENTRETI = :NRINVENTRETI
            ORDER BY P.NMPRODUTO ASC";

        $params = array(
            'NRINVENTRETI' => $nrinvent,
            'CDFILIAL' => $cdfilial,
        );

        // $query = $this->getEntityManager()->getConnection()->prepare($sql);
        // $query->execute($params);
        // $result = $query->fetchAll();
        // return $result;
        return $this->createAndExecuteQueryAndBuildResponseData($this->getEntityManager(), $sql, $params);
    }
    
    public function retornaProdutosInventariosPeriodo($nrorg, $cdfilial, $dtinicio, $dtfinal) {
        $sql = "SELECT ROWNUM, CDFILIAL, NRINVENTRETI, NRITINVENTRETI, QTITINVENTRETI, I.CDPRODUTO,
            P.NMPRODUTO, P.CDARVPROD, I.CDOPERINCLUSAO, I.DTINCLUSAO, I.CDOPERULTATU, I.DTULTATU,
            CDFILIAL || NRINVENTRETI || NRITINVENTRETI AS KEY
            FROM ITEMINVENTRETI I, PRODUTO P
            WHERE P.CDPRODUTO = I.CDPRODUTO
            AND INSTR (:CDFILIAL, I.CDFILIAL) <> 0
            AND I.NRINVENTRETI IN (
                SELECT NRINVENTRETI FROM INVENTARIORETI
                WHERE INSTR(:CDFILIAL, CDFILIAL) <> 0 
                AND DTINVENTRETI BETWEEN TO_DATE(:DATA_INICIAL, 'DD/MM/YYYY') AND TO_DATE(:DATA_FINAL, 'DD/MM/YYYY')
            )
            ORDER BY P.NMPRODUTO ASC";
        $params = array(
            // 'NRINVENTRETI' => $nrinvent,
            'CDFILIAL' => $cdfilial,
            'DATA_FINAL' => $dtfinal,
            'DATA_INICIAL' => $dtinicio,
        );
        return $this->createAndExecuteQueryAndBuildResponseData($this->getEntityManager(), $sql, $params);  
    }

}