<?php
/**
 * Created by Luiz Filho
 */

namespace SCM\Model\Repository;

use Doctrine\ORM\EntityRepository;
use SCM\Traits\TQuery;

class Solicitacao extends EntityRepository {
    
    use TQuery;

    public function retornaDatasEntregaSolicitacao($nrorg, $filiais, $dataInicial, $dataFinal) {
        $dataSet = array();
	  	$sql = "SELECT
            DISTINCT TO_CHAR(ITEMSOLI.DTPROGENTR, 'DD/MM/YYYY') AS DATA,
            TO_CHAR(ITEMSOLI.DTPROGENTR, 'DD/MM/YYYY - DAY', 'NLS_DATE_LANGUAGE= ''BRAZILIAN PORTUGUESE''') AS DATA_EXTENSO,
            ITEMSOLI.CDFILSOLI
            FROM ITEMSOLI
            INNER JOIN PRODUTO ON PRODUTO.CDPRODUTO = ITEMSOLI.CDPRODUTO
            INNER JOIN (SELECT NVL(IDEXPSOLCMP, 'N') IDEXPSOLCMP, F.CDFILIAL
            FROM FILIAL F 
                LEFT JOIN PARASUPRI P ON (NVL(F.CDFILCENTCMP, F.CDFILIAL) = P.CDFILIAL)
                WHERE F.CDFILIAL = :CDFILIAL) PARAM ON PARAM.CDFILIAL = ITEMSOLI.CDFILSOLI
            WHERE ITEMSOLI.DTPROGENTR BETWEEN TO_DATE(:DATA_INICIAL, 'DD/MM/YYYY') AND TO_DATE(:DATA_FINAL, 'DD/MM/YYYY')
            AND ITEMSOLI.CDFILSOLI = :CDFILIAL
            AND (ITEMSOLI.IDSOLIAPROV = PARAM.IDEXPSOLCMP OR 'S' = PARAM.IDEXPSOLCMP)
            ORDER BY TO_DATE(DATA, 'DD/MM/YYYY') ASC";

	  	$params = array(
            'DATA_INICIAL' => $dataInicial,
            'DATA_FINAL' => $dataFinal,
            'CDFILIAL' => ''
		);

        foreach ($filiais as $filial) {
            $params['CDFILIAL'] = $filial;
            $query = $this->getEntityManager()->getConnection()->prepare($sql);
            $query->execute($params);
            $dataSet = array_merge($dataSet, $query->fetchAll());
        }
        return $dataSet;
        
    }

    public function retornaProdutosSolicitacao($nrorg, $cdfilial, $dataInicial, $dataFinal) {
        $parametro = $this->parametroSolicitacaoAprovada($cdfilial, $nrorg);

        $sql = "SELECT ROWNUM, ITEMSOLI.CDFILLAN||' - '||ITEMSOLI.NRSOLICMP AS CDFILICONCATNRSOL, ITEMSOLI.CDFILLAN, ITEMSOLI.CDFILSOLI, ITEMSOLI.NRSOLICMP, PRODUTO.CDPRODUTO, PRODUTO.CDARVPROD,
                NMPRODUTO, SGUNIDADE, QTSOLIPROD, TO_CHAR(DTPROGENTR,'DD/MM/YYYY') AS DTPROGENTR,
                CDFILIOPELOG, IDSOLIAPROV, NULL AS QTAJUSTADA, NULL AS DSJUSTOPER,
                ITEMSOLI.IDITMSOLEXTRA, MOVSOLEXT.DSJUSTSOLEXT,
                ITEMSOLI.CDFILSOLI AS CDFILIAL,
                ITEMSOLI.CDFILSOLI || ITEMSOLI.CDFILLAN || ITEMSOLI.NRSOLICMP || ITEMSOLI.CDPRODUTO || TO_CHAR(DTPROGENTR,'DDMMYYYY') AS KEY,
                TO_CHAR(DTPROGENTR,'DD/MM/YYYY') AS DTINICIAL, TO_CHAR(DTPROGENTR,'DD/MM/YYYY') AS DTFINAL 
                FROM ITEMSOLI
                INNER JOIN SOLICITA ON  ITEMSOLI.CDFILLAN = SOLICITA.CDFILLANCA AND ITEMSOLI.CDFILSOLI = SOLICITA.CDFILSOLI AND ITEMSOLI.NRSOLICMP = SOLICITA.NRSOLICMP
                INNER JOIN PRODUTO ON PRODUTO.CDPRODUTO = ITEMSOLI.CDPRODUTO 
                LEFT JOIN MOVSOLEXT ON MOVSOLEXT.CDFILLANCEXT = ITEMSOLI.CDFILLAN AND MOVSOLEXT.CDFILSOLEXT = ITEMSOLI.CDFILSOLI AND MOVSOLEXT.NRSOLIMOVEXT = ITEMSOLI.NRSOLICMP AND TRUNC(MOVSOLEXT.DTMOVSOLEXT) = TRUNC(SOLICITA.DTSOLCMP)
                WHERE ITEMSOLI.DTPROGENTR BETWEEN TO_DATE(:DATA_INICIAL, 'DD/MM/YYYY') AND TO_DATE(:DATA_FINAL, 'DD/MM/YYYY')
                AND INSTR(:CDFILIAL, ITEMSOLI.CDFILSOLI) <> 0
                AND (ITEMSOLI.IDSOLIAPROV =:IDSOLIAPROV OR 'S' =:IDSOLIAPROV)
                ORDER BY NMPRODUTO, TO_DATE(DTPROGENTR,'DD/MM/YYYY')";
                // AND ITEMSOLI.CDFILSOLI =:CDFILIAL 
        $params = array(
            'DATA_INICIAL' => $dataInicial,
            'DATA_FINAL' => $dataFinal,
            'IDSOLIAPROV' => $parametro,
            'CDFILIAL' => $cdfilial
        );
        
        // if (!is_array($cdfilial)) $cdfilial = [$cdfilial];
        // foreach ($cdfilial as $filial) {
        //     $params['CDFILIAL'] = $filial;
        //     $query = $this->getEntityManager()->getConnection()->prepare($sql);
        //     $query->execute($params);
        //     $dataSet = array_merge($dataSet, $query->fetchAll());
        // }
        return $this->createAndExecuteQueryAndBuildResponseData($this->getEntityManager(), $sql, $params);
    }

    private function parametroSolicitacaoAprovada($cdfilial, $nrorg) {

        $sql = "SELECT NVL(IDEXPSOLCMP, 'N') IDEXPSOLCMP
            FROM FILIAL F 
                LEFT JOIN PARASUPRI P ON (NVL(F.CDFILCENTCMP, F.CDFILIAL) = P.CDFILIAL)
                WHERE F.CDFILIAL = :CDFILIAL";

        $params = array(
            'CDFILIAL' => $cdfilial,
        );

        $query = $this->getEntityManager()->getConnection()->prepare($sql);
        $query->execute($params);
        $result = $query->fetchAll();
        return isset($result[0]) ? $result[0]['IDEXPSOLCMP'] : 'N';
    }

    public function retornaSolicitacoes($nrorg, $filiais, $dataInicial, $dataFinal) {
        $dataSet = array();
        $sql = "SELECT CDFILLANCA,
              CDFILSOLI,
              NRSOLICMP,
              TO_CHAR(DTSOLCMP, 'DD/MM/YYYY') AS DTSOLCMP
            FROM SOLICITA
            WHERE CDFILSOLI = :CDFILIAL
            AND DTSOLCMP BETWEEN TO_DATE(:DATA_INICIAL, 'DD/MM/YYYY') AND TO_DATE(:DATA_FINAL, 'DD/MM/YYYY')
            ORDER BY DTSOLCMP DESC,
              NRSOLICMP DESC";

        $params = array(
            'DATA_INICIAL' => $dataInicial,
            'DATA_FINAL' => $dataFinal,
            'CDFILIAL' => '',
        );
        foreach ($filiais as $filial) {
            $params['CDFILIAL'] = $filial;
            $query = $this->getEntityManager()->getConnection()->prepare($sql);
            $query->execute($params);
            $dataSet = array_merge($dataSet, $query->fetchAll());
        }
        return $dataSet;
    }

    public function retornaProximoCodigoSolicitacao($nrorg, $filiais) {
        $dataSet = array();
        $sql = "SELECT
              LPAD((NVL(MAX(NRSOLICMP),0) + 1),10,'0') PROXIMOCODIGO,
              CDFILSOLI
            FROM SOLICITA
            WHERE CDFILSOLI = :CDFILIAL
            AND LENGTH(NRSOLICMP) = 10 
            GROUP BY CDFILSOLI";

        $params = array(
            'CDFILIAL' => '',
        );
        foreach ($filiais as $filial) {
            $params['CDFILIAL'] = $filial;
            $query = $this->getEntityManager()->getConnection()->prepare($sql);
            $query->execute($params);
            $dataSet = array_merge($dataSet, $query->fetchAll());
        }
        return $dataSet;
    }

    public function retornaNumeroSolicitacao($row) {

        $sql = "SELECT
              LPAD((NVL(MAX(NRSOLICMP),0) + 1),10,'0') PROXIMOCODIGO,
              CDFILSOLI
            FROM SOLICITA
            WHERE CDFILSOLI = :CDFILSOLI
            AND CDFILLANCA = :CDFILLANCA
            AND LENGTH(NRSOLICMP) = 10 
            GROUP BY CDFILSOLI";

        $params = array(
            'CDFILSOLI' => $row['CDFILSOLI'],
            'CDFILLANCA' => $row['CDFILLANCA'],
        );

        $query = $this->getEntityManager()->getConnection()->prepare($sql);
        $query->execute($params);
        $result = $query->fetchAll();

        if(empty($result)) {
            $result = array(
                array(
                    'PROXIMOCODIGO' => '0000000001',
                    'CDFILSOLI' => $row['CDFILSOLI'],
                )
            );
        }

        return $result[0]['PROXIMOCODIGO'];
        
    }

    public function retornaCdfiliopelog($produto, $cdfilcentcmp) {
        if ($this->consolidaSoliCompParaOL($cdfilcentcmp)) {

            $sql = "SELECT MAX(CDFILIOPELOG) AS CDFILIOPELOG
                FROM PARAMPRODSUP
                WHERE CDFILIAL = :CDFILIAL
                AND CDPRODUTO = :CDPRODUTO";

            $params = array(
                'CDPRODUTO' => $produto['CDPRODUTO'],
                'CDFILIAL' => $produto['CDFILSOLI'],
            );

            $query = $this->getEntityManager()->getConnection()->prepare($sql);
            $query->execute($params);
            $result = $query->fetchAll();

            if (empty($result)) {
                return null;
            } else {
                return $result[0]['CDFILIOPELOG'];
            }
        } else {
            return null;
        }

    }

    private function consolidaSoliCompParaOL($cdfilial) {
        $sql = "SELECT IDUTILCONSSOLIOL FROM PARASUPRI WHERE CDFILIAL = :CDFILIAL";
        $params = array('CDFILIAL' => $cdfilial);

        $query = $this->getEntityManager()->getConnection()->prepare($sql);
        $query->execute($params);
        $result = $query->fetchAll();

        if (!empty($result)) {
            if ($result[0]['IDUTILCONSSOLIOL'] == 'S') {
                return true;
            } else {
                return false;
            }
        } else {
            return false;
        }
    }
}