<?php

namespace SCM\Services;

use SCM\Traits\TQuery;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class RequisicaoRetirada {
    
    use TQuery;

    private $entityManager;
    private $environment;
    
    public function __construct(\Doctrine\ORM\EntityManager $entityManager, $environment) {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
    }
    
    public function retornaInsumosPlanejados($centrais, $dtInicial, $dtFinal, $nrconsplan) {
        $sql = "SELECT C.CDFILCENTPME || C.NRCONSPLAN || C.CDTPCARDPME || C.CDFILICPLA || C.CDSERVCPLA || TO_CHAR(C.DTCONSPLAN, 'DDMMYYYY')  
                    || C.IDCONSPLAN || C.CDPRODCPLA AS KEY,
                    C.CDFILCENTPME, C.NRCONSPLAN, C.CDTPCARDPME, C.CDFILICPLA, C.CDSERVCPLA,  TO_CHAR(C.DTCONSPLAN, 'DD/MM/YYYY') AS DTCONSPLAN,
            	   C.IDCONSPLAN, C.CDPRODCPLA, P.NMPRODUTO, P.SGUNIDADE, P.CDARVPROD,
            	   QTCONSPLAN AS QTPREVCONSPLAN,
            	   QTCONSPLAN AS QTPREVISTA,
            	   QTCONSPLAN AS QTCONSUMIDA,
            	   DECODE(C.IDCONSPLAN, 'F', C.QTCONSPLAN, C.QTCONSPLAN / QTCOMESTCPLA) AS QTPERCAPITA,
            	   DECODE(C.IDCONSPLAN, 'F', C.QTCONSPLAN, (C.QTCONSPLAN / QTCOMESTCPLA) * NVL(:P_COMENSAIS, 0)) AS QTDTOTAL,
                    CASE WHEN R.CDFILIAL IS NOT NULL THEN 'S' ELSE 'N' END IDRETIRADA,
            	   'N' AS IDINSUMPLAN,
            	   CD.NRCARDPME, CD.NRCARDIAPME
              FROM CONSPLAN C
             INNER JOIN TIPOCARD TP ON TP.CDTIPOCARD = C.CDTPCARDPME
            					   AND TP.CDFILCENTPME = C.CDFILCENTPME
             INNER JOIN FILIAL F ON F.CDFILIAL = C.CDFILICPLA
             INNER JOIN SERVICO S ON S.CDFILIAL = C.CDFILICPLA
            					 AND S.CDSERVFILI = C.CDSERVCPLA
             INNER JOIN PRODUTO P ON P.CDPRODUTO = C.CDPRODCPLA
             LEFT JOIN CARDIAPME CD ON CD.CDFILCENTPME = C.CDFILCENTPME AND CD.CDSERVFILI = C.CDSERVCPLA AND CD.CDFILIAL = C.CDFILICPLA AND CD.DTCARDIAPME = C.DTCONSPLAN
             LEFT JOIN (SELECT CA.DTCARDAPIO, RE.CDFILIAL, CA.CDSERVFILI
              FROM CARDADIA CA
              LEFT JOIN RETIRADA RE ON RE.NRCARDAPIO = CA.NRCARDAPIO
                                   AND RE.CDFILIAL = CA.CDFILIAL) R ON R.CDFILIAL = C.CDFILICPLA AND R.CDSERVFILI = C.CDSERVCPLA AND R.DTCARDAPIO = C.DTCONSPLAN
             WHERE INSTR(:P_NRCONSPLAN, C.NRCONSPLAN) <> 0
               AND INSTR(:P_CDFILCENTPME, C.CDFILCENTPME) <> 0
               AND C.DTCONSPLAN BETWEEN :P_DTINICIAL AND :P_DTFINAL";
               
            //   TP.NMTIPOCARD, F.NMFILIAL, S.NMSERVICO,
                //  C.NRCONSPLAN = :P_NRCONSPLAN
                // AND C.CDFILCENTPME = :P_CDFILCENTPME
        
        $params = array(
            ':P_COMENSAIS' => null,
            ':P_RETIRADA' => null,
            ':P_NRCONSPLAN' => $nrconsplan,
            ':P_CDFILCENTPME' => $centrais,
            ':P_DTINICIAL' => $dtInicial,
            ':P_DTFINAL' => $dtFinal,
        );
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $params);
    }
    
    public function getCardiapme($cdfilcentpme, $cdservfili, $cdfilial, $dtcardiapme) {
        $sql = "SELECT NRCARDPME, NRCARDIAPME FROM CARDIAPME CA WHERE CA.CDFILIAL = :CDFILIAL AND CA.CDSERVFILI = :CDSERVFILI 
                    AND CA.CDFILCENTPME = :CDFILCENTPME AND DTCARDIAPME = :DTCARDIAPME";
                    
        $query = $this->entityManager->getConnection()->prepare($sql);
        $params = array(
            'CDFILCENTPME' => $cdfilcentpme,
            'CDSERVFILI' => $cdservfili,
            'CDFILIAL' => $cdfilial,
            'DTCARDIAPME' => $dtcardiapme,
		);
        $query->execute($params);
        $result = $query->fetchAll(\PDO::FETCH_ASSOC);
        return $result;
    }
    
    public function updateEfetivoDia($cdfilcentpme, $nrcardpme, $nrcardiapme, $cdservfili, $cdfilial, $dtcardiapme, $qtcomefecpme, $qtcliefecpme) {
        $sql = "UPDATE CARDIAPME SET QTCOMEFECPME = :QTCOMEFECPME, QTCLIEFECPME = :QTCLIEFECPME
                WHERE CDFILCENTPME = :CDFILCENTPME
                    AND NRCARDPME = :NRCARDPME
                    AND NRCARDIAPME = :NRCARDIAPME
                    AND CDSERVFILI = :CDSERVFILI
                    AND CDFILIAL = :CDFILIAL
                    AND DTCARDIAPME = :DTCARDIAPME";
                    
        $sth = $this->entityManager->getConnection()->prepare($sql);
        $params = array(
            'CDFILCENTPME' => $cdfilcentpme,
            'NRCARDPME' => $nrcardpme,
            'NRCARDIAPME' => $nrcardiapme,
            'CDSERVFILI' => $cdservfili,
            'CDFILIAL' => $cdfilial,
            'DTCARDIAPME' => $dtcardiapme,
            'QTCOMEFECPME' => $qtcomefecpme,
            'QTCLIEFECPME' => $qtcliefecpme,
		);
		$sth->execute($params);
    }
    
    public function procedure($rows){
        try {
          $nrorg = $this->environment->getNrOrg();
          $cdoperador = $this->environment->getCdOperador(); 
          
          if(empty($rows['PRODINICIAL'])) $cdprodIni = '0';
          else $cdprodIni = $rows['PRODINICIAL'];
          
          if(empty($rows['PRODFINAL'])) $cdprodFin = 'ZZZZZZZZZ';
          else $cdprodFin = str_pad($rows['PRODFINAL'], 10, 'Z');
          
          
          if(empty($rows['SERVICO']) and !(empty($rows['GRUPOSERVICO']))) {
            $rows['SERVICO'] = $this->getServicosGrupo($rows);
            $servProce = $rows['SERVICO'];
          } else if (empty($rows['SERVICO']) and empty($rows['GRUPOSERVICO'])) {
            $rows['SERVICO'] = null;
            $servProce = null;
          } else {
            if (is_array($rows['SERVICO'])) {
              $servicos = [];
              $unidServ = [];
              foreach($rows['SERVICO'] as $row){
                $concatServ = $row['CDSERVFILI'];
                $concatUnid = $row['CDFILIAL'];
                array_push($servicos, $concatServ);
                array_push($unidServ, $concatUnid);
              }
              $rows['SERVICO'] = join("_", $servicos);
              $rows['UNIDADE'] = join("_", array_unique($unidServ));
            } 
            
            $servProce = $rows['SERVICO'];
          }
          
          if ($rows['GRUPFILI'] == NULL) {
            if (empty($rows['UNIDADE'])) {
              $rows['UNIDADE'] = $this->getUnidades($rows);
              $unidade = $rows['UNIDADE'];
            } else {
              $rows['UNIDADE'] = is_array($rows['UNIDADE']) ? join("_", $rows['UNIDADE']) : $rows['UNIDADE'];
              $unidade = $rows['UNIDADE'];
            }
          } else {
            if (empty($rows['UNIDADE'])) {
              $rows['UNIDADE'] = $rows['UNIDADE'] = $this->getUnidadesGrupo($rows, $nrorg);  
              $unidade = $rows['UNIDADE'];
            }
            else {
              $rows['UNIDADE'] = is_array($rows['UNIDADE']) ? join("_", $rows['UNIDADE']) : $rows['UNIDADE'];
              $unidade = $rows['UNIDADE'];
            }
          }
          
          $tipoCardapio = null;
          if (isset($rows['TIPOCARD'])) {
            if (is_array($rows['TIPOCARD'])) {
              foreach($rows['TIPOCARD'] as $tpCard) {
                $tipoCardapio .= $tpCard['CDTIPOCARD']."_";
              }
            } else {
              $tipoCardapio = $rows['TIPOCARD'];
            }
          }
          
          $connection = $this->entityManager->getConnection();     
          
          for ($i = 0; $i <= strlen($unidade); $i+=3000) {
    
            $unid = substr($unidade, $i, 3000);
            
            $procedureCons = new StoredProcedure($connection, 'CONSUMO_PLANEJADO_PME'); 
            
            $procedureCons->addParam(new Param('P_CDFILCENTPME', Param::PARAM_INPUT, $rows['CDFILCENTPME'], Param::PARAM_TYPE_STR));
            $procedureCons->addParam(new Param('P_CDTIPOCARD',   Param::PARAM_INPUT, $tipoCardapio,         Param::PARAM_TYPE_STR));
            $procedureCons->addParam(new Param('P_CDFILIAL',     Param::PARAM_INPUT, $unid,                 Param::PARAM_TYPE_STR));
            $procedureCons->addParam(new Param('P_CDSERVFILI',   Param::PARAM_INPUT, $servProce,            Param::PARAM_TYPE_STR));
            $procedureCons->addParam(new Param('P_DTINICIAL',    Param::PARAM_INPUT, $rows['DTDE'],         Param::PARAM_TYPE_STR));
            $procedureCons->addParam(new Param('P_DTFINAL',      Param::PARAM_INPUT, $rows['DTATE'],        Param::PARAM_TYPE_STR));
            $procedureCons->addParam(new Param('P_IDARQUIVO',    Param::PARAM_INPUT, 'A',                   Param::PARAM_TYPE_STR));
            $procedureCons->addParam(new Param('P_IDPRODUTOS',   Param::PARAM_INPUT, 'CXX',                 Param::PARAM_TYPE_STR));
            $procedureCons->addParam(new Param('P_IDSEPPRATO',   Param::PARAM_INPUT, 'N',                   Param::PARAM_TYPE_STR));
            $procedureCons->addParam(new Param('P_PEACRESCIMO',  Param::PARAM_INPUT, '0',                   Param::PARAM_TYPE_STR));
            $procedureCons->addParam(new Param('P_IDARREDOND',   Param::PARAM_INPUT, 'N',                   Param::PARAM_TYPE_STR));
            $procedureCons->addParam(new Param('P_NRCONSPLAN',   Param::PARAM_INPUT, $rows['NRCONSPLAN'],   Param::PARAM_TYPE_STR));
            $procedureCons->addParam(new Param('P_CDPRODUTOINI', Param::PARAM_INPUT, $cdprodIni,            Param::PARAM_TYPE_STR));
            $procedureCons->addParam(new Param('P_CDPRODUTOFIN', Param::PARAM_INPUT, $cdprodFin,            Param::PARAM_TYPE_STR));
            $procedureCons->addParam(new Param('P_CDOPERADOR',   Param::PARAM_INPUT, $cdoperador,           Param::PARAM_TYPE_STR));
            $procedureCons->execute();
          }
          return array('error' => false, 'response' => array('result' => array($rows)));
        } catch(\Exception $e) {
            return array('error' => true, 'errorMessage' => $e->getMessage().$e->getTraceAsString());
        }
    }
    
    private function getServicosGrupo($rows) {
      $nrorg = $this->environment->getNrOrg();
      $servicos = [];
      
      if(is_array($rows['GRUPOSERVICO'])) $grupServ = join("_", $rows['GRUPOSERVICO']);
      else $grupServ = $rows['GRUPOSERVICO'];

      $query = "SELECT S.CDSERVFILI, S.NMSERVICO
                  FROM SERVICO S
                 WHERE ( ( INSTR(:CDGRUPSERV, S.CDGRUPSERV) <> 0 ) OR ( :CDGRUPSERV = 'T' ) ) 
                   AND ( S.CDFILIAL = :CDFILCENTPME ) AND S.IDTIPOSERV = 'M'
                 ORDER BY S.CDSERVFILI";
      
      $params = array(
        "CDFILCENTPME" => $rows["CDFILCENTPME"],
        "CDGRUPSERV"   => $rows['GRUPOSERVICO']
      );
      $stmt = $this->entityManager->getConnection()->prepare($query);
  	  $stmt->execute($params);
      $result = $stmt->fetchAll();
      
      $servGrupo = [];
      foreach($result as $serv){
        array_push($servGrupo, $serv['CDSERVFILI']);
      }
      $resultado = join("_", $servGrupo);
      return $resultado;
    }
    
    public function getUnidades($rows){
      $cdoperador = $this->environment->getCdOperador(); 
      $unidades = [];
      //$query = "SELECT DISTINCT FI.CDFILIAL, FI.NMFILIAL FROM FILIAL FI, FILIOPER FO WHERE (FI.IDFILCENTPME = 'N') AND (FI.CDFILCENTPME = :CDFILCENTPME) AND (FI.CDFILIAL = FO.CDFILIAL) AND (INSTR(:CDOPERADOR, FO.CDOPERADOR) <> 0 OR :CDOPERADOR IS NULL) ORDER BY FI.CDFILIAL";
      $query = "SELECT DISTINCT FI.CDFILIAL, FI.NMFILIAL FROM FILIAL FI, FILIOPER FO WHERE (FI.IDFILCENTPME = 'N') AND (FI.CDFILCENTPME = :CDFILCENTPME) AND (FI.CDFILIAL = FO.CDFILIAL) AND :CDOPERADOR = FO.CDOPERADOR ORDER BY FI.CDFILIAL";
      $params = array(
        "CDFILCENTPME" => $rows["CDFILCENTPME"],
        "CDOPERADOR"   => $cdoperador
      );
      $stmt = $this->entityManager->getConnection()->prepare($query);
	  $stmt->execute($params);
      $result = $stmt->fetchAll();

      foreach($result as $unid){
        array_push($unidades, $unid['CDFILIAL']);
      }
      $resultado = join("_", $unidades);
      
      return $resultado;
    }
    
    public function getUnidadesGrupo($rows, $nrorg){
      if (is_array($rows['GRUPFILI'])) $grupoUnid = $rows['GRUPFILI']['CDGRUPFILI'];
      else $grupoUnid = $rows['GRUPFILI'];
      
      $cdoperador = $this->environment->getCdOperador(); 
      $unidGrupo = [];
      $query = "SELECT DISTINCT F.CDFILIAL, F.NMFILIAL, F.CDFILIAL || ' - ' || F.NMFILIAL AS DESCRICAO, F.CDFILCENTPME 
                  FROM FILIAL F 
                  LEFT JOIN FILGRUPFILI FL ON ( FL.CDGRUPFILI =:CDGRUPFILI OR :CDGRUPFILI = 'T' ) AND FL.CDFILIAL = F.CDFILIAL 
                 WHERE ( FL.CDGRUPFILI IS NOT NULL OR :CDGRUPFILI = 'T' ) AND F.CDFILCENTPME =:CDFILCENTPME 
                 ORDER BY CDFILIAL";
      $params = array(
        "CDFILCENTPME" => $rows["CDFILCENTPME"],
        "CDGRUPFILI"   => $grupoUnid
      );
      $stmt = $this->entityManager->getConnection()->prepare($query);
	  $stmt->execute($params);
      $result = $stmt->fetchAll();

      foreach($result as $unid){
        array_push($unidGrupo, $unid['CDFILIAL']);
      }
      $resultado = join("_", $unidGrupo);
      return $resultado;
    }
}