<?php

namespace Tecfoodlib\Modules\PLA\Controllers;

class Retirada {
    protected $connection;
    protected $environment;
    protected $newCodeNT;
    protected $Utils;
    protected $prefixo;
    protected $retiradaPorPrato;
    protected $controlaAlmoxarifado;
    protected $controlaLocal;
    protected $controlaLote;
    protected $visualizaSaldo;
    protected $precisao;
    protected $ValidacoesEstoqueService;
    protected $logIntegracao;
    protected $cancelaEmErroDeProduto;
    protected $geraDaRequisicao;
    protected $exigeProdutos;
    protected $restauraRetiradaSeProdutoSemEstoque;
    protected $abortEstqAtIrregularities;
    
    public function __construct($connection, $environment, $newCodeNT, $Utilities, $ValidacoesEstoqueService){
        $this->connection = $connection;
        $this->environment = $environment;
        $this->newCodeNT = $newCodeNT;
        $this->Utils = $Utilities;
        $this->ValidacoesEstoqueService = $ValidacoesEstoqueService;
        
        $this->prefixo = "";
        $this->retiradaPorPrato = false;
        $this->controlaAlmoxarifado = false;
        $this->controlaLocal = false;
        $this->controlaLote = false;
        $this->visualizaSaldo = false;
        $this->precisao = 5;
        $this->cancelaEmErroDeProduto = false;
        $this->geraDaRequisicao = true;
        $this->exigeProdutos = false;
        $this->restauraRetiradaSeProdutoSemEstoque = true;
        $this->abortEstqAtIrregularities = true;
    }
    //Inclusão de mensagens de erro / aviso
    private function incluiMensagem($msg, $erro, $complemento=null){
        $this->Utils->incluiMensagem($msg, $erro, $complemento);
    }
    //Consulta e retorna uma lista de serviços para a filial (todos habilitados para retiradas)
    private function consultaServicos($cdfilial){
        $query = "
            SELECT CDFILIAL, CDSERVFILI, NMSERVICO
              FROM SERVICO
             WHERE CDFILIAL = :CDFILIAL
               AND IDSERVATIV <> 'N'
             ORDER BY CDSERVFILI
        ";
        $params = [
            "CDFILIAL" => $cdfilial
        ];
        $prepared = $this->connection->prepare($query);
        $prepared->execute($params);
        return $prepared->fetchAll();
    }
    //Consulta dados da filial relacionados à retirada
    private function consultaFilial($cdfilial){
        $query = "
            SELECT IDCNTRRESC
              FROM FILIAL
             WHERE CDFILIAL = :CDFILIAL
        ";
        $prepared = $this->connection->prepare($query);
        $params = ["CDFILIAL" => $cdfilial];
        $prepared->execute($params);
        return $prepared->fetch();
    }
    //Consulta informações do serviço indicado
    private function consultaServicoparaRetirada($cdfilial, $cdservfili){
        $query = "
            SELECT CDFILIAL, CDSERVFILI, NMSERVICO, IDUTRETIPRAT
              FROM SERVICO
             WHERE CDFILIAL = :CDFILIAL
               AND CDSERVFILI = :CDSERVFILI
               AND IDSERVATIV <> 'N'
        ";
        $params = [
            "CDFILIAL" => $cdfilial,
            "CDSERVFILI" => $cdservfili
        ];
        $prepared = $this->connection->prepare($query);
        $prepared->execute($params);
        return $prepared->fetch();
    }
    //Consulta dados do cardápio indicado
    private function consultaCardapio($cdfilial, $cdservfili, $dtcardapio){
        $query = "
            SELECT CDFILIAL, CDSERVFILI, NRCARDAPIO
              FROM CARDADIA
             WHERE CDFILIAL = :CDFILIAL
               AND CDSERVFILI = :CDSERVFILI
               AND DTCARDAPIO = :DTCARDAPIO
        ";
        $params = [
            "CDFILIAL" => $cdfilial,
            "CDSERVFILI" => $cdservfili,
            "DTCARDAPIO" => $dtcardapio
        ];
        $prepared = $this->connection->prepare($query);
        $prepared->execute($params);
        return $prepared->fetch();
    }
    //Insere cardápio no banco de dados
    private function insereCardapio($cdfilial, $cdservfili, $dtcardapio){
        $nrcardapio = $this->newCodeNT->getCodeByTableName('CARDAPIO' . $cdfilial, $this->environment->getNrOrg());
        $query = "
            INSERT INTO CARDADIA
            (CDFILIAL, NRCARDAPIO, CDSERVFILI, CDOPERADOR,
             DTCARDAPIO,DTMESCARD,
             NRORG, NRORGULTATU, NRORGINCLUSAO,
             DTULTATU, DTINCLUSAO,
             CDOPERULTATU, CDOPERINCLUSAO)
            VALUES
            (:CDFILIAL, :NRCARDAPIO, :CDSERVFILI, :CDOPERADOR,
             :DTCARDAPIO, TRUNC(TO_DATE(:DTCARDAPIO), 'MM'),
             :NRORG, :NRORG, :NRORG,
             SYSDATE, SYSDATE,
             :CDOPERADORLOGADO, :CDOPERADORLOGADO)
        ";
        $params = [
            "CDFILIAL" => $cdfilial,
            "NRCARDAPIO" => $nrcardapio,
            "CDSERVFILI" => $cdservfili,
            "CDOPERADOR" => $this->environment->getCdOperador(),
            "DTCARDAPIO" => $dtcardapio,
            "NRORG" => $this->environment->getNrOrg(),
            "CDOPERADORLOGADO" => $this->environment->getCdOperador()
        ];
        $prepared = $this->connection->prepare($query);
        $prepared->execute($params);
        return $nrcardapio;
    }
    //Consulta o número de cardápio indicado, se não existir cria um
    private function consultaOuInsereNrCardapio($cdfilial, $cdservfili, $dtcardapio){
        $cardapio = $this->consultaCardapio($cdfilial, $cdservfili, $dtcardapio);
        if(!$cardapio){
            return $this->insereCardapio($cdfilial, $cdservfili, $dtcardapio);
        }
        else{
            return $cardapio['NRCARDAPIO'];
        }
    }
    //Consulta dados da retirada indicada
    private function consultaRetirada($cdfilial, $nrcardapio, $dtretirada, $nrreti=null){
        if(empty($nrcardapio)) return false;
        $query = "";
        $params = [];
        if(!empty($nrreti)) {
            $query = "
                SELECT R.CDFILIAL, R.NRCARDAPIO, C.CDSERVFILI, R.NRRETIRADA, TO_CHAR(C.DTCARDAPIO, 'DD/MM/YYYY') AS DTCARDAPIO,
                TO_CHAR(R.DTRETIRADA, 'DD/MM/YYYY') AS DTRETIRADA, R.CDFILANCESTQ, R.NRLANCESTQ, 'N' AS RETI_DIA_DIF, R.IDRETIPRAT
                  FROM RETIRADA R
                       INNER JOIN CARDADIA C ON C.CDFILIAL = R.CDFILIAL AND C.NRCARDAPIO = R.NRCARDAPIO
                 WHERE R.CDFILIAL = :CDFILIAL
                   AND R.NRCARDAPIO = :NRCARDAPIO
                   AND R.NRRETIRADA = :NRRETIRADA
            ";
            $params = [
                "CDFILIAL" => $cdfilial,
                "NRCARDAPIO" => $nrcardapio,
                "NRRETIRADA" => $nrreti
            ];
        }
        else {
            $query = "
                SELECT R.CDFILIAL, R.NRCARDAPIO, C.CDSERVFILI, R.NRRETIRADA, TO_CHAR(C.DTCARDAPIO, 'DD/MM/YYYY') AS DTCARDAPIO,
                TO_CHAR(R.DTRETIRADA, 'DD/MM/YYYY') AS DTRETIRADA, R.CDFILANCESTQ, R.NRLANCESTQ,
                CASE WHEN R.DTRETIRADA <> :DTRETIRADA THEN 'S' ELSE 'N' END RETI_DIA_DIF, R.IDRETIPRAT
                  FROM RETIRADA  R
                       INNER JOIN CARDADIA C ON C.CDFILIAL = R.CDFILIAL AND C.NRCARDAPIO = R.NRCARDAPIO
                 WHERE R.CDFILIAL = :CDFILIAL
                   AND R.NRCARDAPIO = :NRCARDAPIO
            ";
            $params = [
                "CDFILIAL" => $cdfilial,
                "NRCARDAPIO" => $nrcardapio,
                "DTRETIRADA" => $dtretirada
            ];
        }
        $prepared = $this->connection->prepare($query);
        $prepared->execute($params);
        $resultado = $prepared->fetch();
        if(!$resultado) {
            return false;
        }
        if($resultado['RETI_DIA_DIF'] == 'S') {
            $this->incluiMensagem('retiJaExisteDiaDif', true, $resultado['DTRETIRADA']);
        }
        return $resultado;
    }
    //Insere novo lançamento em estoque no banco de dados
    private function insereLancamentoEstoque($cdfilial, $dtlancestq, $dslancestq){
            $nrlancestq = $this->newCodeNT->getCodeByTableName('LANCTOEST'.$cdfilial, $this->environment->getNrOrg());
            $query = "
                INSERT INTO LANCTOESTOQ
                (CDFILIAL, CDFILIMOVI, NRLANCESTQ,
                 DTLANCESTQ, DSLANCESTQ, IDTPLANCTO,
                 IDALTCUSMED, IDSTATUSCTBEST, CDOPERADOR, 
                 NRORG, NRORGULTATU, NRORGINCLUSAO,
                 DTULTATU, DTINCLUSAO,
                 CDOPERULTATU, CDOPERINCLUSAO)
                VALUES
                (:CDFILIAL, :CDFILIAL, :NRLANCESTQ,
                 :DTLANCESTQ, :DSLANCESTQ, :IDTPLANCTO,
                 :IDALTCUSMED, :IDSTATUSCTBEST, :CDOPERADOR,
                 :NRORG, :NRORG, :NRORG,
                 SYSDATE, SYSDATE,
                 :CDOPERADORLOGADO, :CDOPERADORLOGADO)
            ";
            $params = [
                "CDFILIAL" => $cdfilial,
                "NRLANCESTQ" => $nrlancestq,
                "DTLANCESTQ" => $dtlancestq,
                "DSLANCESTQ" => $dslancestq,
                "IDTPLANCTO" => '3',
                "IDALTCUSMED" => 'N',
                "IDSTATUSCTBEST" => 'P',
                "CDOPERADOR" => $this->environment->getCdOperador(),
                "NRORG" => $this->environment->getNrOrg(),
                "CDOPERADORLOGADO" => $this->environment->getCdOperador()
            ];
            $prepared = $this->connection->prepare($query);
            $prepared->execute($params);
            return $nrlancestq;
        }
    //Apaga lançamento de estoque
    private function excluiLancamentoEstoque($cdfilial, $nrlancestq){
        $query = "
            DELETE FROM LANCTOESTOQ
             WHERE CDFILIAL = :CDFILIAL
               AND NRLANCESTQ = :NRLANCESTQ
        ";
        $params = [
            "CDFILIAL" => $cdfilial,
            "NRLANCESTQ" => $nrlancestq
        ];
        $prepared = $this->connection->prepare($query);
        $prepared->execute($params);    
    }
    //Insere nova retirada no banco de dados
    private function insereRetirada($cdfilial, $nrcardapio, $dtretirada, $nrlancestq, $nrrequextrre){
        $nrretirada = $this->newCodeNT->getCodeByTableName('RETIRADA' . $cdfilial, $this->environment->getNrOrg());
        $retiradaPorPrato = ($this->retiradaPorPrato) ? 'S' : 'N';
        $query = "
            INSERT INTO RETIRADA
            (CDFILIAL, NRCARDAPIO, NRRETIRADA, NRLANCESTQ,
             DTRETIRADA, NRREQUEXTRRE, CDFILANCESTQ, IDRETIPRAT,
             NRORG, NRORGULTATU, NRORGINCLUSAO,
             DTULTATU, DTINCLUSAO,
             CDOPERULTATU, CDOPERINCLUSAO)
            VALUES 
            (:CDFILIAL, :NRCARDAPIO, :NRRETIRADA, :NRLANCESTQ,
             :DTRETIRADA, :NRREQUEXTRRE, :CDFILIAL, :IDRETIPRAT,
             :NRORG, :NRORG, :NRORG,
             SYSDATE, SYSDATE,
             :CDOPERADORLOGADO, :CDOPERADORLOGADO)
        ";
        $params = [
            "CDFILIAL" => $cdfilial,
            "NRCARDAPIO" => $nrcardapio,
            "NRRETIRADA" => $nrretirada,
            "NRLANCESTQ" => $nrlancestq,
            "DTRETIRADA" => $dtretirada,
            "NRREQUEXTRRE" => $nrrequextrre,
            "IDRETIPRAT" => $retiradaPorPrato,
            "NRORG" => $this->environment->getNrOrg(),
            "CDOPERADORLOGADO" => $this->environment->getCdOperador()
        ];
        $prepared = $this->connection->prepare($query);
        $prepared->execute($params);    
    }
    //Apaga a retirada do banco
    private function removeRetirada($cdfilial, $nrcardapio, $nrretirada, $cdfilancestq, $nrlancestq){
        $query = "
            DELETE FROM RETIRADA
             WHERE CDFILIAL = :CDFILIAL
               AND NRCARDAPIO = :NRCARDAPIO
               AND NRRETIRADA = :NRRETIRADA
        ";
        $params = [
            "CDFILIAL" => $cdfilial,
            "NRCARDAPIO" => $nrcardapio,
            "NRRETIRADA" => $nrretirada
        ];
        $prepared = $this->connection->prepare($query);
        $prepared->execute($params);
        $this->excluiLancamentoEstoque($cdfilancestq, $nrlancestq);
    }
    //Verifica se existe requisição, e se a mesma é por tipo de prato
    private function verificaExistePrevReti($cdfilial, $cdservfili, $dtcardapio, $nrrequextrre) {
        $query = "
            SELECT MAX(P.IDPREVTPPRAT) AS IDPREVTPPRAT
            FROM PREVRETI P
            WHERE P.CDFILIAL = :CDFILIAL
            AND P.CDSERVFILI = :CDSERVFILI
            AND P.DTPREVRETI = :DTPREVRETI
            AND P.NRREQUEXTR = :NRREQUEXTRRE
        ";
        $params = [
            "CDFILIAL" => $cdfilial,
            "CDSERVFILI" => $cdservfili,
            "DTPREVRETI" => $dtcardapio,
            "NRREQUEXTRRE" => $nrrequextrre
        ];
        $prepared = $this->connection->prepare($query);
        $prepared->execute($params);
        return $prepared->fetch();    
    }
    //Consulta a retirada indicada, inclui caso não exista 
    private function consultaOuInsereRetirada($cdfilial, $cdservfili, $dtretirada, $dtcardapio, $dslancestq, $nrrequextrre){
        $nrcardapio = $this->consultaOuInsereNrCardapio($cdfilial, $cdservfili, $dtcardapio);
        $retirada = $this->consultaRetirada($cdfilial, $nrcardapio, $dtretirada);
        if(!$retirada){
            //Se tiver requisição, verifica se a requisição é por tipo de prato
            if($this->geraDaRequisicao) {
                $existePrevreti = $this->verificaExistePrevReti($cdfilial, $cdservfili, $dtcardapio, $nrrequextrre);
                if($existePrevreti) {
                    $this->retiradaPorPrato = ($existePrevreti['IDPREVTPPRAT'] == 'S');
                }
            }
            $nrlancestq = $this->insereLancamentoEstoque($cdfilial, $dtretirada, $dslancestq);
            $this->insereRetirada($cdfilial, $nrcardapio, $dtretirada, $nrlancestq, $nrrequextrre);
            $retirada = $this->consultaRetirada($cdfilial, $nrcardapio, $dtretirada);
            $retirada['NOVA'] = true;
        } else {
            $retirada['NOVA'] = false;
        }
        return $retirada;
    }
    //Consulta parêmetros da paramgeral relacionados à retirada
    private function consultaParamgeral(){
        $query = "SELECT IDUTILDTPROC FROM PARAMGERAL";
        $prepared = $this->connection->prepare($query);
        $prepared->execute();
        return $prepared->fetch();
    }
    //Consulta parêmetros da paramestoque relacionados à retirada
    private function consultaParamestoque(){
        $query = "
            SELECT PE.IDLANCDTMEN,
                   PE.IDLANCDTMAI,
                   NVL(PE.NRDIASSAID, 0) AS NRDIASSAID,
                   NVL(PE.NRDIASSAIDPD, 0) AS NRDIASSAIDPD
              FROM PARAMESTOQUE PE
        ";
        $prepared = $this->connection->prepare($query);
        $prepared->execute();
        return $prepared->fetch();
    }
    //Consulta parêmetros da paramfilial relacionados à retirada
    private function consultaParamfilial($cdfilial){
        $query = "
            SELECT PF.IDLANCDTMENFIL,
                   PF.IDLANCDTMAIFIL,
                   PF.IDUTILALMOX,
                   PF.IDUTILLCESTQ,
                   PF.IDUTILLOTE,
                   PF.DTPROCESSA,
                   NVL(PF.NRDIASSAIDFIL, 0) AS NRDIASSAIDFIL,
                   NVL(PF.NRDIASSAIDPDFIL, 0) AS NRDIASSAIDPDFIL,
                   PF.IDPERMVISPOSEST
              FROM PARAMFILIAL PF
             WHERE PF.CDFILIAL = :CDFILIAL
        ";
        $prepared = $this->connection->prepare($query);
        $prepared->execute(['CDFILIAL' => $cdfilial]);
        $paramfilial = $prepared->fetch();
        if($paramfilial) {
            $this->controlaAlmoxarifado = ($paramfilial['IDUTILALMOX'] == 'S' ) ? true : false;
            $this->controlaLocal = ($paramfilial['IDUTILLCESTQ'] == 'S' ) ? true : false;
            $this->controlaLote = ($paramfilial['IDUTILLOTE'] == 'S') ? true : false;
            $this->visualizaSaldo = ($paramfilial['IDPERMVISPOSEST'] == 'S') ? true : false;
        }
        return $paramfilial;
    }
    //Consulta parâmetros da paramplaneja relacionados à retirada
    private function consultaParamPlaneja($cdfilial){
        $query = "
            SELECT IDUTREQUPRAT, CDTPRETIPADR
              FROM PARAMPLANEJA
             WHERE CDFILIAL = :CDFILIAL
        ";
        $prepared = $this->connection->prepare($query);
        $prepared->execute(['CDFILIAL' => $cdfilial]);
        return $prepared->fetch();    
    }
    //Consulta informações do produto indicado juntamente com dados da prodfili
    private function consultaProduto($cdproduto, $cdfilial){
        $query = "
        SELECT P.CDPRODUTO, P.CDPRODESTO, NVL(P.IDPRODATIVPR, 'S') AS IDPRODATIVPR, PF.CDFILIAL,
               NVL(PF.IDPRODATIV, 'A') AS IDPRODATIV, NVL(PF.IDCNTRESTOQ, IDCNTRESTOQ) AS IDCNTRESTOQ
          FROM PRODUTO P
               LEFT JOIN PRODFILI PF ON PF.CDPRODUTO = P.CDPRODUTO 
                                    AND PF.CDFILIAL = :CDFILIAL
         WHERE P.CDPRODUTO = :CDPRODUTO
        ";
        $prepared = $this->connection->prepare($query);
        $params = [
            'CDPRODUTO' => $cdproduto,
            'CDFILIAL' => $cdfilial
        ];
        $prepared->execute($params);
        return $prepared->fetch();
    }
    //Consulta o tipo de prato indicado no banco de dados
    private function consultaTipoPrato($cdtipoprat){
        $query = "
            SELECT CDTIPOPRAT
              FROM TIPOPRAT
             WHERE CDTIPOPRAT = :CDTIPOPRAT
        ";
        $prepared = $this->connection->prepare($query);
        $prepared->execute(['CDTIPOPRAT' => $cdtipoprat]);
        return $prepared->fetch();
    }
    //Consulta o prato indicado no banco de dados
    private function consultaPrato($cdfilauxprat, $cdprato){
        $query = "
            SELECT CDFILAUXPRAT, CDPRATO
              FROM PRATO
             WHERE CDFILAUXPRAT = :CDFILAUXPRAT
               AND CDPRATO = :CDPRATO
               AND CDNVPRATO = (SELECT MAX(CDNVPRATO) FROM PRATO)
        ";
        $prepared = $this->connection->prepare($query);
        $params = [
            'CDFILAUXPRAT' => $cdfilauxprat,
            'CDPRATO' => $cdprato
        ];
        $prepared->execute($params);
        return $prepared->fetch();
    }
    //Consulta item específico da retirada existente
    private function consultaItemReti($cdfilial, $nrcardapio, $nrretirada, $cdproduto, $nrlancestq, $cdtipoprat, 
                                      $cdfilauxprat, $cdprato, $cdalmoxarife, $cdlocalestoq, $nrloteestq, $nrsublote, $cdfilancestq){
        $retiradaPorPrato = ($this->retiradaPorPrato) ? 'S' : 'N';
        $query = "
            SELECT I.CDFILIAL, I.NRCARDAPIO, I.NRRETIRADA, I.NRSEQUITRETI, I.CDPRODUTO,
                   I.QTPREVISRETI, I.QTEXTRARETI, I.QTDEVOLVRETI, I.QTRETIRADA, I.NRSEQUITEM,
                   I.QTSALDORETIRADA, IL.QTLANCTOEST, IL.QTTOTLANCTO, IL.VRUNILANCTO, IL.VRTOTLANCTO,
                   IL.IDEVOLESTOQ
              FROM ITEMRETI I
                   LEFT JOIN ITLANCTOEST IL ON IL.CDFILIAL = I.CDFIITLANEST AND
                                               IL.NRLANCESTQ = I.NRLANCESTQ AND
                                               IL.NRSEQUITEM = I.NRSEQUITEM
             WHERE I.CDFILIAL = :CDFILIAL
               AND I.NRCARDAPIO = :NRCARDAPIO
               AND I.NRRETIRADA = :NRRETIRADA
               AND I.CDPRODUTO = :CDPRODUTO
               AND I.CDFIITLANEST = :CDFIITLANEST
               AND I.NRLANCESTQ = :NRLANCESTQ
               AND ((:RETIRADAPORPRATO = 'N') OR (I.CDTIPOPRAT = :CDTIPOPRAT))
               AND ((:RETIRADAPORPRATO = 'N') OR (I.CDFILAUXPRAT = :CDFILAUXPRAT))
               AND ((:RETIRADAPORPRATO = 'N') OR (I.CDPRATO = :CDPRATO))
               AND IL.CDALMOXARIFE = :CDALMOXARIFE
               AND IL.CDLOCALESTOQ = :CDLOCALESTOQ
               AND IL.NRLOTEESTQ = :NRLOTEESTQ
               AND IL.NRSUBLOTE = :NRSUBLOTE
        ";
        $prepared = $this->connection->prepare($query);
        $params = [
            'CDFILIAL' => $cdfilial,
            'NRCARDAPIO' => $nrcardapio,
            'NRRETIRADA' => $nrretirada,
            'CDPRODUTO' => $cdproduto,
            'CDFIITLANEST' => $cdfilancestq,
            'NRLANCESTQ' => $nrlancestq,
            'RETIRADAPORPRATO' => $retiradaPorPrato,
            'CDTIPOPRAT' => $cdtipoprat,
            'CDFILAUXPRAT' => $cdfilauxprat,
            'CDPRATO' => $cdprato,
            'CDALMOXARIFE' => $cdalmoxarife,
            'CDLOCALESTOQ' => $cdlocalestoq,
            'NRLOTEESTQ' => $nrloteestq,
            'NRSUBLOTE' => $nrsublote
        ];
        $prepared->execute($params);
        return $prepared->fetch();    
    }
    //Consulta os itens de retirada. É possível filtrar apenas os que apresentaram erro de estoque
    private function consultaItensReti($cdfilial, $nrcardapio, $nrretirada, $comErroEstoque=false){
        $erroEmEstoque = ($comErroEstoque) ? 'N' : null;
        $query = "
            SELECT I.NRSEQUITRETI 
              FROM ITEMRETI I
                   LEFT JOIN ITLANCTOEST IL ON IL.CDFILIAL = I.CDFIITLANEST AND
                                               IL.NRLANCESTQ = I.NRLANCESTQ AND
                                               IL.NRSEQUITEM = I.NRSEQUITEM
             WHERE I.CDFILIAL = :CDFILIAL
               AND I.NRCARDAPIO = :NRCARDAPIO
               AND I.NRRETIRADA = :NRRETIRADA
               AND ((IL.IDEVOLESTOQ = :IDEVOLESTOQ) OR (:IDEVOLESTOQ IS NULL))
        ";
        $prepared = $this->connection->prepare($query);
        $params = [
            'CDFILIAL' => $cdfilial,
            'NRCARDAPIO' => $nrcardapio,
            'NRRETIRADA' => $nrretirada,
            'IDEVOLESTOQ' => $erroEmEstoque
        ];
        $prepared->execute($params);
        return $prepared->fetchAll();
    }
    //Consulta a próxima numeração de item da retirada
    private function consultaProxNrItem($cdfilial, $nrcardapio, $nrretirada){
        $query = "
            SELECT MAX(NRSEQUITRETI) AS NRSEQUITRETI
              FROM ITEMRETI
             WHERE CDFILIAL = :CDFILIAL
               AND NRCARDAPIO = :NRCARDAPIO
               AND NRRETIRADA = :NRRETIRADA
        ";
        $prepared = $this->connection->prepare($query);
        $params = [
            'CDFILIAL' => $cdfilial,
            'NRCARDAPIO' => $nrcardapio,
            'NRRETIRADA' => $nrretirada
        ];
        $prepared->execute($params);
        $numero = $prepared->fetch();     
        $numero = $numero['NRSEQUITRETI'];
        if(empty($numero)){
            $numero = 0;
        }
        else{
            try{
                $numero = intval($numero) + 1;
            }
            catch(\Exception $e){
                $numero = 0;
            }
        }
        return $numero;
    }
    //Retorna data da última posição do produto (considera como limite um dia após a data da retirada)
    private function retornaUltimaDataPosicaoEstoque($cdfilial, $dtretirada, $cdproduto, $cdalmoxarife, $cdlocalestoq, $nrloteestq, $nrsublote){
        $query = "
            SELECT MAX(DTPOSIESTQ) AS DTPOSIESTQ
              FROM POSESTDIA
             WHERE CDFILIAL = :CDFILIAL
               AND CDPRODUTO = :CDPRODUTO
               AND CDALMOXARIFE = :CDALMOXARIFE
               AND CDLOCALESTOQ = :CDLOCALESTOQ
               AND NRLOTEESTQ = :NRLOTEESTQ
               AND NRSUBLOTE = :NRSUBLOTE
               AND DTPOSIESTQ <= TO_DATE(:DTPOSIESTQ) + 1
        ";
        $prepared = $this->connection->prepare($query);
        $params = [
            'CDFILIAL' => $cdfilial,
            'CDPRODUTO' => $cdproduto,
            'CDALMOXARIFE' => $cdalmoxarife,
            'CDLOCALESTOQ' => $cdlocalestoq,
            'NRLOTEESTQ' => $nrloteestq,
            'NRSUBLOTE' => $nrsublote,
            'DTPOSIESTQ' => $dtretirada
        ];
        $prepared->execute($params);
        $posestdia = $prepared->fetch();
        return $posestdia['DTPOSIESTQ'];
        
    }
    //Consulta e retorna o saldo na retirada do produto no inicio do dia posterior a retirada (ou seja considerando todos os lançamentos já feitos no dia)
    private function consultaSaldoDia($cdfilial, $dtretirada, $cdproduto, $cdalmoxarife, $cdlocalestoq, $nrloteestq, $nrsublote){
        $dtultpos = $this->retornaUltimaDataPosicaoEstoque($cdfilial, $dtretirada, $cdproduto, $cdalmoxarife, $cdlocalestoq, $nrloteestq, $nrsublote);
        $query = "
            SELECT QTESTOQDIA
              FROM POSESTDIA
             WHERE CDFILIAL = :CDFILIAL
               AND CDPRODUTO = :CDPRODUTO
               AND CDALMOXARIFE = :CDALMOXARIFE
               AND CDLOCALESTOQ = :CDLOCALESTOQ
               AND NRLOTEESTQ = :NRLOTEESTQ
               AND NRSUBLOTE = :NRSUBLOTE
               AND DTPOSIESTQ = :DTPOSIESTQ
        ";
        $prepared = $this->connection->prepare($query);
        $params = [
            'CDFILIAL' => $cdfilial,
            'CDPRODUTO' => $cdproduto,
            'CDALMOXARIFE' => $cdalmoxarife,
            'CDLOCALESTOQ' => $cdlocalestoq,
            'NRLOTEESTQ' => $nrloteestq,
            'NRSUBLOTE' => $nrsublote,
            'DTPOSIESTQ' => $dtultpos
        ];
        $prepared->execute($params);
        $posestdia = $prepared->fetch();
        if(!$posestdia) return 0;
        return $posestdia['QTESTOQDIA'];
        
    }
    //Obtém custo de estoque do produto
    private function consultaCustoEstoque($cdfilial, $cdproduto, $dtlancto, $cdalmoxarife, $cdlocalestoq, $nrloteestq, $nrsublote){
        $query = "
            SELECT CUSTO_PRODUTO_LCEST(
            :CDFILIAL, :CDPRODUTO, TO_NUMBER(NVL(CDTIPCUSTEST, '3')),
            :DTPOSICAO, SYSDATE, :CDALMOXARIFE, :CDLOCALESTOQ,
            :NRLOTEESTQ, :NRSUBLOTE) AS CUSTO
              FROM PARAMFILIAL
             WHERE CDFILIAL = :CDFILIAL
        ";
        $prepared = $this->connection->prepare($query);
        $params = [
            'CDFILIAL' => $cdfilial,
            'CDPRODUTO' => $cdproduto,
            'DTPOSICAO' => $dtlancto,
            'CDALMOXARIFE' => $cdalmoxarife,
            'CDLOCALESTOQ' => $cdlocalestoq,
            'NRLOTEESTQ' => $nrloteestq,
            'NRSUBLOTE' => $nrsublote
        ];
        $prepared->execute($params);
        $custo = $prepared->fetch();
        if(!$custo) return 0.0;
        return $custo['CUSTO'];
    }
    //Consulta almoxarifado no banco de dados
    private function consultaAlmoxarifado($cdfilial, $cdalmoxarife){
        $query = "
        SELECT CDALMOXARIFE 
          FROM ALMOXARIFADO
         WHERE CDFILIAL = :CDFILIAL
           AND CDALMOXARIFE = :CDALMOXARIFE
        ";
        $prepared = $this->connection->prepare($query);
        $params = [
            "CDFILIAL" => $cdfilial,
            "CDALMOXARIFE" => $cdalmoxarife
        ];
        $prepared->execute($params);
        return $prepared->fetch();
    }
    //Consulta local de estoque no banco de dados
    private function consultaLocalEstoque($cdfilial, $cdalmoxarife, $cdlocalestoq){
        $query = "
            SELECT CDLOCALESTOQ
              FROM LOCALESTOQUE
             WHERE CDFILIAL = :CDFILIAL
               AND CDALMOXARIFE = :CDALMOXARIFE
               AND CDLOCALESTOQ = :CDLOCALESTOQ
        ";
        $prepared = $this->connection->prepare($query);
        $params = [
            "CDFILIAL" => $cdfilial,
            "CDALMOXARIFE" => $cdalmoxarife,
            "CDLOCALESTOQ" => $cdlocalestoq
        ];
        $prepared->execute($params);
        return $prepared->fetch();    
    }
    //Consulta almoxarifado no banco de dados
    private function consultaLote($cdfilial, $dtretirada, $cdproduto, $nrloteestq, $nrsublote){
        $query = "
        SELECT NRLOTEESTQ
          FROM LOTE
         WHERE CDFILIAL = :CDFILIAL
           AND NRLOTEESTQ = :NRLOTEESTQ
           AND NRSUBLOTE = :NRSUBLOTE
           AND CDPRODUTO = :CDPRODUTO
           AND TO_DATE(:DTRETIRADA) <= DTVALILOTE
        ";
        $prepared = $this->connection->prepare($query);
        $params = [
            "CDFILIAL" => $cdfilial,
            "NRLOTEESTQ" => $nrloteestq,
            "NRSUBLOTE" => $nrsublote,
            "CDPRODUTO" => $cdproduto,
            "DTRETIRADA" => $dtretirada
        ];
        $prepared->execute($params);
        return $prepared->fetch();
    }
    //Verifica se o parâmetro de filial é valido
    private function validaFilial($cdfilial){
        if(!$cdfilial){
            $this->incluiMensagem("unidNInf", true);
            return false;
        }
        $filial = $this->consultaFilial($cdfilial);
        if(!$filial){
            $this->incluiMensagem("unidNCad", true);
            return false;
        }
        else if ($filial['IDCNTRRESC'] == 'S'){
            $this->incluiMensagem("unidRescind", true);
            return false;
        }
        return true;
    }
    //Verifica se o parâmetro de serviço é valido
    private function validaServico($cdfilial, $cdservfili){
        if(empty($cdservfili)){
            $this->incluiMensagem("servNInf", true);
            return false;
        }
        $servico = $this->consultaServicoparaRetirada($cdfilial, $cdservfili);
        if(!$servico){
            $this->incluiMensagem("servNCad", true);
            return false;
        }
        if($servico['IDUTRETIPRAT'] == 'S'){
            $this->retiradaPorPrato = true;
        }
        return true;
    }
    //Valida o valor da data de cardápio informada
    private function validaDtCard($dtcardapio){
        if(empty($dtcardapio)){
            $this->incluiMensagem('dtCardNInf', true);
            return false;
        }
        if(!$this->Utils->dataValida($dtcardapio)){
            $this->incluiMensagem('dtCardInv', true);
            return false;
        }
        return true;
    }
    //Valida o valor da data de retirada informada
    private function validaDtReti($dtretirada){
        if(empty($dtretirada)){
            $this->incluiMensagem('dtRetiNInf', true);
            return false;
        }
        if(!$this->Utils->dataValida($dtretirada)){
            $this->incluiMensagem('dtRetiInv', true);
            return false;
        }
        return true;
    }
    //Verifica se o parâmetro de data é valido e se está no período permitido
    private function validaPeriodo($cdfilial, $dtretirada, $paramfilial){
        $dtretirada = $this->Utils->retornaDatetime($dtretirada);
        $paramgeral = $this->consultaParamgeral();
        $paramestoque = $this->consultaParamestoque();
        
        $idutildtproc = ($paramgeral) ? $paramgeral['IDUTILDTPROC'] : 'N';
        $dtprocessa = $this->Utils->retornaDatetime(date('d/m/Y'));

        $idlancdtmen = 'N';
        $idlancdtmai = 'N';
        $nrdiassaid = 0;
        $nrdiassaidpd = 0;
        if($paramestoque){
            $idlancdtmen = $paramestoque['IDLANCDTMEN'];
            $idlancdtmai = $paramestoque['IDLANCDTMAI'];
            $nrdiassaid = $paramestoque['NRDIASSAID'];
            $nrdiassaidpd = $paramestoque['NRDIASSAIDPD'];
        }
        
        $idlancdtmenfil = 'N';
        $nrdiassaidfil = 0;
        $idlancdtmaifil = 'N';
        $nrdiassaidpdfil = 0;
        
        if($paramfilial){
            if($idutildtproc = 'S' && !empty($paramfilial['DTPROCESSA'])){
                $dtprocessa = $this->Utils->retornaDatetime($paramfilial['DTPROCESSA']);
            }
            $idlancdtmenfil = $paramfilial['IDLANCDTMENFIL'];
            $nrdiassaidfil = $paramfilial['NRDIASSAIDFIL'];
            $idlancdtmaifil = $paramfilial['IDLANCDTMAIFIL'];
            $nrdiassaidpdfil = $paramfilial['NRDIASSAIDPDFIL'];
        }
        
        $diasAnterior = 0;
        if($idlancdtmenfil = 'S'){
            $diasAnterior = $nrdiassaidfil;
        } else if ($idlancdtmen = 'S'){
            $diasAnterior = $nrdiassaid;
        }
        
        $diasPosterior = 0;
        
        if($idlancdtmaifil = 'S'){
            $diasPosterior = $nrdiassaidpdfil;
        } else if ($idlancdtmai = 'S'){
            $diasPosterior = $nrdiassaidpd;
        }
        $dataAnterior = clone $dtprocessa;
        $dataAnterior->sub(new \DateInterval('P'.$diasAnterior.'D'));
        $dataPosterior = clone $dtprocessa;
        $dataPosterior->add(new \DateInterval('P'.$diasPosterior.'D'));
        if (($dtretirada < $dataAnterior) || ($dtretirada > $dataPosterior)){
            $intervalo = [
                "inicio" => $dataAnterior->format('d/m/Y'),
                "fim" => $dataPosterior->format('d/m/Y')
            ];
            $this->incluiMensagem("dtretiForaIntervalo", true, $intervalo);
            return false;
        }
        
        return true;
    }
    //Verifica parametrizações de planejamento para a retirada. Atribui o tipo de retirada padrão.
    private function validaParamPlaneja($cdfilial, &$cdtpretipadr){
        $cdtpretipadr = null;
        $paramplaneja = $this->consultaParamPlaneja($cdfilial);
        if(!$paramplaneja){
            $this->incluiMensagem("tpRetiNCad", true);
            return true;
        } else{
            $idObrigaRetiradaPrato = ($paramplaneja['IDUTREQUPRAT'] == 'S') ? true : false;
            if($idObrigaRetiradaPrato && !$this->retiradaPorPrato){
                $this->incluiMensagem("retiPratoExigida", true);
                return false;
            }
            $cdtpretipadr = $paramplaneja['CDTPRETIPADR'];
            if(empty($cdtpretipadr)){
                $this->incluiMensagem("tpRetiNCad", true);
            }
            return true;
        }
    }
    //Verifica se a lista de produtos informada é válida
    private function validaListaProdutos($produtos){
        if(!is_array($produtos)){
            $this->incluiMensagem("prodNInf", true);
            return false;
        }
        if(count($produtos) == 0){
            $this->incluiMensagem("prodNInf", true);
            return false;
        }
        return true;
    }
    //Verifica se a descrição do lançamento de estoque é válida
    private function validaDescricaoEstoque($dslancestq){
        if(!$dslancestq){
            $this->incluiMensagem("dsLancNInf", true);
            return false;
        }
        return true;    
    }
    /*Verifica se o produto informado é válido, contém todas as informações para realização da retirada
    Normaliza valores para realização da retirada do produto*/
    private function validaERetornaProduto($produto, $indice, $cdfilial, $nrcardapio, $nrretirada, $dtretirada, $nrlancestq, $cdtpretipadr, $cdfilancestq){
        $produtoValido = true;
        
        $cdproduto = $this->Utils->valor($produto, 'CDPRODUTO');
        $qtPrevista = $this->Utils->valor($produto, 'QTPREVISRETI', 0.0);
        $qtPrevista = round($qtPrevista, $this->precisao);
        $qtExtra = $this->Utils->valor($produto, 'QTEXTRARETI', 0.0);
        $qtExtra = round($qtExtra, $this->precisao);
        $qtDevolvida = $this->Utils->valor($produto, 'QTDEVOLVRETI', 0.0);
        $qtDevolvida = round($qtDevolvida, $this->precisao);
        $qtRetirada = $this->Utils->valor($produto, 'QTRETIRADA', 0.0);
        $qtRetirada = round($qtRetirada, $this->precisao);
        $cdfilauxprat = $this->Utils->valor($produto, 'CDFILAUXPRAT');
        $cdprato = $this->Utils->valor($produto, 'CDPRATO');
        $cdtipoprat = $this->Utils->valor($produto, 'CDTIPOPRAT');
        
        $cdalmoxarife = $this->Utils->valor($produto, 'CDALMOXARIFE', ' ');
        $cdalmoxarife = (empty($cdalmoxarife)) ? ' ' : $cdalmoxarife;
        $cdlocalestoq = $this->Utils->valor($produto, 'CDLOCALESTOQ', ' ');
        $cdlocalestoq = (empty($cdlocalestoq)) ? ' ' : $cdlocalestoq;
        $nrloteestq = $this->Utils->valor($produto, 'NRLOTEESTQ', ' ');
        $nrloteestq = (empty($nrloteestq)) ? ' ' : $nrloteestq;
        $nrsublote = $this->Utils->valor($produto, 'NRSUBLOTE', ' ');
        $nrsublote = (empty($nrsublote)) ? ' ' : $nrsublote;
        
        $sobrepor = $this->Utils->valor($produto, 'sobrepor', true);
        $excluir = $this->Utils->valor($produto, 'excluir', false);
        
        $nrsequitreti = null;
        $nrsequitem = null;
        $qtPrevistaJaLancada = 0;
        $qtExtraJaLancada = 0;
        $qtDevolvidaJaLancada = 0;
        $qtRetiradaJaLancada = 0;
        $qtSaldoRetiradaJaLancado = 0;
        $qtlanctoest = 0;
        $qttotlancto = 0;
        $vrunilancto = 0;
        $vrtotlancto = 0;
        $idevolestoq = 'N';
        
        $produtoNovo = true;
        
        $produtoTeknisa = $this->consultaProduto($produto['CDPRODUTO'], $cdfilial);
        if(!$produtoTeknisa){
            $this->incluiMensagem("prodNCad", $this->cancelaEmErroDeProduto, $indice);
            $produtoValido = false;
        }
        $produtoEstoque = ($produtoTeknisa['CDPRODUTO'] == $produtoTeknisa['CDPRODESTO']);
        $produtoControlaEstoque = ($produtoTeknisa['IDCNTRESTOQ'] == 'S');
        $ativo = ($produtoTeknisa['IDPRODATIV'] == 'A') || ($produtoTeknisa['IDPRODATIVPR'] == 'S');
        if(!$ativo){
            $this->incluiMensagem("prodNAtivo", $this->cancelaEmErroDeProduto, $indice);
            $produtoValido = false;
        }
        if(!$produtoEstoque){
            $this->incluiMensagem("prodNProdEstoque", $this->cancelaEmErroDeProduto, $indice);
            $produtoValido = false;
        }
        if(!$produtoControlaEstoque){
            $this->incluiMensagem("prodNControlaEst", $this->cancelaEmErroDeProduto, $indice);
            $produtoValido = false;
        }
        
        //Se item já existe na base, registra os dados antigos (que serão substituídos)
        $itemreti = $this->consultaItemReti($cdfilial, $nrcardapio, $nrretirada, $cdproduto, $nrlancestq, 
                                            $cdtipoprat, $cdfilauxprat, $cdprato,
                                            $cdalmoxarife, $cdlocalestoq, $nrloteestq, $nrsublote, $cdfilancestq);
        
        if($itemreti){
            $produtoNovo = false;
            $nrsequitreti = $itemreti['NRSEQUITRETI'];
            $nrsequitem = $itemreti['NRSEQUITEM'];
            $qtPrevistaJaLancada = ($itemreti['QTPREVISRETI'] == null) ? 0 : $itemreti['QTPREVISRETI'];
            $qtPrevista = $qtPrevistaJaLancada;
            $qtExtraJaLancada = ($itemreti['QTEXTRARETI'] == null) ? 0 : $itemreti['QTEXTRARETI'];
            $qtDevolvidaJaLancada = ($itemreti['QTDEVOLVRETI'] == null) ? 0 : $itemreti['QTDEVOLVRETI'];
            $qtRetiradaJaLancada = ($itemreti['QTRETIRADA'] == null) ? 0 : $itemreti['QTRETIRADA'];
            $qtSaldoRetiradaJaLancado = $itemreti['QTSALDORETIRADA'];
            
            $qtlanctoest = $itemreti['QTLANCTOEST'];
            $qttotlancto = $itemreti['QTTOTLANCTO'];
            $vrunilancto = $itemreti['VRUNILANCTO'];
            $vrtotlancto = $itemreti['VRTOTLANCTO'];
            $idevolestoq = $itemreti['IDEVOLESTOQ'];
        }
        
        if ($excluir && $produtoNovo) {
            $this->incluiMensagem("prodAExclNExiste", $this->cancelaEmErroDeProduto, $indice);
            $produtoValido = false;
        }
        
        if($qtExtra <= 0 && $qtDevolvida <= 0 && $qtRetirada <= 0){
            $this->incluiMensagem("prodSemQtInf", $this->cancelaEmErroDeProduto, $indice, $this->cancelaEmErroDeProduto, $indice);
            $produtoValido = false;
        }
        
        if ($qtRetirada !== round($qtPrevista + $qtExtra - $qtDevolvida, $this->precisao)) {
            $this->incluiMensagem("prodQtRetiInfErr", $this->cancelaEmErroDeProduto, $indice, $this->cancelaEmErroDeProduto, $indice);
            $produtoValido = false;
        }
        
         //Valida tipo de prato e prato se o serviço obriga retirada por prato
        if($this->retiradaPorPrato){
            if(empty($cdtipoprat)){
                $this->incluiMensagem("tpPratNInf", $this->cancelaEmErroDeProduto, $indice);
                $produtoValido = false;
            }else{
                $tipoprat = $this->consultaTipoPrato($cdtipoprat);
                if(!$tipoprat){
                    $this->incluiMensagem("tpPratNCad", $this->cancelaEmErroDeProduto, $indice);
                    $produtoValido = false;
                }
            }
            
            if(empty($cdfilauxprat) || empty($cdprato)){
                $this->incluiMensagem("pratNInf", $this->cancelaEmErroDeProduto, $indice);
                $produtoValido = false;
            }else{
                $prato = $this->consultaPrato($cdfilauxprat, $cdprato);
                if(!$prato){
                    $this->incluiMensagem("pratNCad", $this->cancelaEmErroDeProduto, $indice);
                    $produtoValido = false;
                }
            }
        }
        
        if($this->controlaAlmoxarifado) {
            if($cdalmoxarife == ' '){
                $this->incluiMensagem("almoxNInf", $this->cancelaEmErroDeProduto, $indice);
                $produtoValido = false;
            } else{
                $almoxarifado = $this->consultaAlmoxarifado($cdfilial, $cdalmoxarife);
                if(!$almoxarifado){
                    $this->incluiMensagem("almoxNCad", $this->cancelaEmErroDeProduto, $indice);
                    $produtoValido = false;
                }
            }
        }
        
        if($this->controlaLocal) {
            if($cdlocalestoq == ' '){
                $this->incluiMensagem("localNInf", $this->cancelaEmErroDeProduto, $indice);
                $produtoValido = false;
            } else{
                $local = $this->consultaLocalEstoque($cdfilial, $cdalmoxarife, $cdlocalestoq);
                if(!$local){
                    $this->incluiMensagem("localNCad", $this->cancelaEmErroDeProduto, $indice);
                    $produtoValido = false;
                }
            }
        }
        
        if($this->controlaLote) {
            if($nrloteestq == ' '){
                $this->incluiMensagem("loteNInf", $this->cancelaEmErroDeProduto, $indice);
                $produtoValido = false;
            } else{
                $lote = $this->consultaLote($cdfilial, $dtretirada, $cdproduto, $nrloteestq, $nrsublote);
                if(!$lote){
                    $this->incluiMensagem("loteNCad", $this->cancelaEmErroDeProduto, $indice);
                    $produtoValido = false;
                }
            }
        }
        
        //Se não sobrepõe, incrementa as quantidades já lançadas para o produto
        if(!$sobrepor){
            $qtExtra = round($qtExtraJaLancada + $qtExtra, $this->precisao);
            $qtDevolvida = round($qtDevolvidaJaLancada + $qtDevolvida, $this->precisao);
        }
        
        $qtRetirada = $qtPrevista - $qtDevolvida + $qtExtra;
        $qtsaldoretirada = $this->consultaSaldoDia($cdfilial, $dtretirada, $cdproduto, $cdalmoxarife, $cdlocalestoq, $nrloteestq, $nrsublote);
        if($idevolestoq == 'S'){
            $qtsaldoretirada = $qtsaldoretirada + $qtRetiradaJaLancada;
        }
        
        // Para detalhamento caso a quantidade retirada seja inválida
        $prodRetiInvalido = [
            'indice' => $indice,
            'QTPREVISRETI_ANT' => $qtPrevistaJaLancada,
            'QTEXTRARETI_ANT' => $qtExtraJaLancada,
            'QTDEVOLVRETI_ANT' => $qtDevolvidaJaLancada,
            'QTRETIRADA_ANT' => $qtRetiradaJaLancada,
            'QTPREVISRETI_NOVA' => $qtPrevista,
            'QTEXTRARETI_NOVA' => $qtExtra,
            'QTDEVOLVRETI_NOVA' => $qtDevolvida,
            'QTRETIRADA_NOVA' => $qtRetirada,
            'QTSALDO' => ($this->visualizaSaldo) ? $qtsaldoretirada : null
        ];
        // Se a retirada for um valor negativo significa que prevista, extra ou devolvida não é valor válido
        if($qtRetirada < 0){
            $this->incluiMensagem("qtRetiInv", $this->cancelaEmErroDeProduto, $prodRetiInvalido);
            $produtoValido = false;
        }
        
        
        if($qtRetirada > $qtsaldoretirada) {
            $this->incluiMensagem("qtRetiMaiorSaldo", $this->cancelaEmErroDeProduto, $prodRetiInvalido);
            if ($this->restauraRetiradaSeProdutoSemEstoque) $produtoValido = false;
        }
        
        //Se o tipo de retirada padrão não foi informado, não segue com a inclusão do produto (mas permite a validação dos dados do mesmo)
        if(empty($cdtpretipadr)){
            $produtoValido = false;
        }
        
        if(!$produtoValido) return false;
        
        return [
            'CDPRODUTO' => $cdproduto,
            'CDFILIAL' => $cdfilial,
            'NRCARDAPIO' => $nrcardapio,
            'NRRETIRADA' => $nrretirada,
            'NRLANCESTQ' => $nrlancestq,
            'DTRETIRADA' => $dtretirada,
            'CDFILAUXPRAT' => $cdfilauxprat,
            'CDPRATO' => $cdprato,
            'CDTIPOPRAT' => $cdtipoprat,
            'CDALMOXARIFE' => $cdalmoxarife,
            'CDLOCALESTOQ' => $cdlocalestoq,
            'CDFILANCESTQ' => $cdfilancestq,
            'NRLOTEESTQ' => $nrloteestq,
            'NRSUBLOTE' => $nrsublote,
            'NRSEQUITRETI' => $nrsequitreti, 
            'NRSEQUITEM' => $nrsequitem,
            'QTPREVISRETI' => $qtPrevista,
            'QTPREVISRETI_ANTERIOR' => $qtPrevistaJaLancada,
            'QTEXTRARETI' => $qtExtra,
            'QTEXTRARETI_ANTERIOR' => $qtExtraJaLancada,
            'QTDEVOLVRETI' => $qtDevolvida,
            'QTDEVOLVRETI_ANTERIOR' => $qtDevolvidaJaLancada,
            'QTRETIRADA' => $qtRetirada,
            'QTRETIRADA_ANTERIOR' => $qtRetiradaJaLancada,
            'CDTPRETIPADR' => $cdtpretipadr,
            'QTSALDORETIRADA' => $qtsaldoretirada,
            'QTSALDORETIRADA_ANTERIOR' => $qtSaldoRetiradaJaLancado,
            'QTLANCTOEST_ANTERIOR' => $qtlanctoest,
            'QTTOTLANCTO_ANTERIOR' => $qttotlancto,
            'VRUNILANCTO_ANTERIOR' => $vrunilancto,
            'VRTOTLANCTO_ANTERIOR' => $vrtotlancto,
            'IDEVOLESTOQ' => 'N',
            'IDEVOLESTOQ_ANTERIOR' => $idevolestoq,
            'INDICE' => $indice,
            'PRODUTO_NOVO' => $produtoNovo,
            'EXCLUIR' => $excluir
        ];
    }
    //Popula array de produtos alterados (verifica duplicidades)
    private function registraProdutosAlteradosSemDuplicidade(&$produtos_aplicados, $produto){
        foreach($produtos_aplicados as $prodAlt){
            if(
                ($prodAlt['CDFILIAL'] == $produto['CDFILIAL']) &&
                ($prodAlt['NRCARDAPIO'] == $produto['NRCARDAPIO']) &&
                ($prodAlt['NRRETIRADA'] == $produto['NRRETIRADA']) &&
                ($prodAlt['CDPRODUTO'] == $produto['CDPRODUTO']) &&
                ($prodAlt['NRLANCESTQ'] == $produto['NRLANCESTQ']) &&
                ($prodAlt['CDTIPOPRAT'] == $produto['CDTIPOPRAT']) &&
                ($prodAlt['CDFILAUXPRAT'] == $produto['CDFILAUXPRAT']) &&
                ($prodAlt['CDPRATO'] == $produto['CDPRATO'])
            ){
                $identProd = $produto['INDICE'];
                $identProdAlt = $prodAlt['INDICE'];
                $duplicidade = [
                    'indice' => $identProd,
                    'outroProdIndice' => $identProdAlt
                ];
                $this->incluiMensagem("prodDuplicado", $this->cancelaEmErroDeProduto, $duplicidade);
                return false;
            }
        }
        $produtos_aplicados[] = $produto;
        return true;
    }
    //Insere item de estoque do lançamento
    private function insereItemEstoque($produto){
        $vrunilancto = $this->consultaCustoEstoque($produto['CDFILIAL'], $produto['CDPRODUTO'], $produto['DTRETIRADA'],
                              $produto['CDALMOXARIFE'], $produto['CDLOCALESTOQ'], $produto['NRLOTEESTQ'], $produto['NRSUBLOTE']);
        $vrtotlancto = $produto['QTRETIRADA'] * $vrunilancto;
        $query = "
            INSERT INTO ITLANCTOEST
            (CDFILIAL, NRLANCESTQ, NRSEQUITEM, CDPRODUTO,
             QTLANCTOEST, IDTIPOMOVI, DTLANCMOVI, IDORDEMCLC,
             QTTOTLANCTO, CDFILIMOVI, CDPRODMOVI, IDEVOLESTOQ,
             CDALMOXARIFE, CDLOCALESTOQ, NRLOTEESTQ, NRSUBLOTE,
             CDTIPORETI, VRUNILANCTO, VRTOTLANCTO,
             NRORG, NRORGULTATU, NRORGINCLUSAO,
             DTULTATU, DTINCLUSAO,
             CDOPERULTATU, CDOPERINCLUSAO)
            VALUES 
            (:CDFILIAL, :NRLANCESTQ, :NRSEQUITEM, :CDPRODUTO,
             :QTLANCTOEST, :IDTIPOMOVI, :DTLANCMOVI, :IDORDEMCLC,
             :QTTOTLANCTO, :CDFILIMOVI, :CDPRODMOVI, :IDEVOLESTOQ,
             :CDALMOXARIFE, :CDLOCALESTOQ, :NRLOTEESTQ, :NRSUBLOTE,
             :CDTIPORETI, :VRUNILANCTO, :VRTOTLANCTO,
             :NRORG, :NRORG, :NRORG,
             SYSDATE, SYSDATE,
             :CDOPERADORLOGADO, :CDOPERADORLOGADO)
        ";
        $params = [
            "CDFILIAL" => $produto['CDFILANCESTQ'],
            "NRLANCESTQ" => $produto['NRLANCESTQ'],
            "NRSEQUITEM" => $produto['NRSEQUITRETI'],
            "CDPRODUTO" => $produto["CDPRODUTO"],
            "QTLANCTOEST" => $produto['QTRETIRADA'],
            "IDTIPOMOVI" => '3',
            "DTLANCMOVI" => $produto['DTRETIRADA'],
            "IDORDEMCLC" => '9',
            "QTTOTLANCTO" => $produto['QTRETIRADA'],
            "CDFILIMOVI" => $produto['CDFILIAL'],
            "CDPRODMOVI" => $produto["CDPRODUTO"],
            "IDEVOLESTOQ" => 'N',
            "CDALMOXARIFE" => $produto["CDALMOXARIFE"],
            "CDLOCALESTOQ" => $produto["CDLOCALESTOQ"],
            "NRLOTEESTQ" => $produto["NRLOTEESTQ"],
            "NRSUBLOTE" => $produto["NRSUBLOTE"],
            "CDTIPORETI" => $produto["CDTPRETIPADR"],
            "VRUNILANCTO" => $vrunilancto,
            "VRTOTLANCTO" => $vrtotlancto,
            "NRORG" => $this->environment->getNrOrg(),
            "CDOPERADORLOGADO" => $this->environment->getCdOperador()
        ];
        $prepared = $this->connection->prepare($query);
        $prepared->execute($params);
    }
    //Atualiza item de estoque do lançamento
    private function atualizaItemEstoque($produto, $restaurando = false){
        $qtlanctoest = $produto['QTLANCTOEST_ANTERIOR'];
        $qttotlancto = $produto['QTTOTLANCTO_ANTERIOR'];
        $vrunilancto = $produto['VRUNILANCTO_ANTERIOR'];
        $vrtotlancto = $produto['VRTOTLANCTO_ANTERIOR'];
        $idevolestoq = $produto['IDEVOLESTOQ_ANTERIOR'];
        if(!$restaurando){
            $qtlanctoest = $produto['QTRETIRADA'];
            $qttotlancto = $produto['QTRETIRADA'];
            $idevolestoq = 'N';
            $vrunilancto = $this->consultaCustoEstoque($produto['CDFILIAL'], $produto['CDPRODUTO'], $produto['DTRETIRADA'],
                                  $produto['CDALMOXARIFE'], $produto['CDLOCALESTOQ'], $produto['NRLOTEESTQ'], $produto['NRSUBLOTE']);
            $vrtotlancto = $produto['QTRETIRADA'] * $vrunilancto;
        }
        $query = "
            UPDATE ITLANCTOEST
               SET QTLANCTOEST = :QTLANCTOEST,
                   QTTOTLANCTO = :QTTOTLANCTO,
                   VRUNILANCTO = :VRUNILANCTO,
                   VRTOTLANCTO = :VRTOTLANCTO,
                   IDEVOLESTOQ = :IDEVOLESTOQ,
                   DTULTATU = SYSDATE,
                   CDOPERULTATU = :CDOPERADORLOGADO
             WHERE CDFILIAL = :CDFILIAL
               AND NRLANCESTQ = :NRLANCESTQ
               AND NRSEQUITEM = :NRSEQUITEM
        ";
        $params = [
            "QTLANCTOEST" => $qtlanctoest,
            "QTTOTLANCTO" => $qttotlancto,
            "VRUNILANCTO" => $vrunilancto,
            "VRTOTLANCTO" => $vrtotlancto,
            "IDEVOLESTOQ" => $idevolestoq,
            "CDFILIAL" => $produto['CDFILANCESTQ'],
            "NRLANCESTQ" => $produto['NRLANCESTQ'],
            "NRSEQUITEM" => $produto['NRSEQUITRETI'],
            "CDOPERADORLOGADO" => $this->environment->getCdOperador()
        ];
        $prepared = $this->connection->prepare($query);
        $prepared->execute($params);       
    }
    //Exclui item de estoque do lançamento
    private function excluiItemEstoque($produto){
        $query = "
            DELETE FROM ITLANCTOEST
             WHERE CDFILIAL = :CDFILIAL
               AND NRLANCESTQ = :NRLANCESTQ
               AND NRSEQUITEM = :NRSEQUITEM
        ";
        $params = [
            "CDFILIAL" => $produto['CDFILANCESTQ'],
            "NRLANCESTQ" => $produto['NRLANCESTQ'],
            "NRSEQUITEM" => $produto['NRSEQUITRETI']
        ];
        $prepared = $this->connection->prepare($query);
        $prepared->execute($params);
    }
    //Exclui todos os itens do lançamento de estoque
    private function excluiTodosItensEstoque($cdfilancestq, $nrlancestq){
        $query = "
            DELETE FROM ITLANCTOEST
             WHERE CDFILIAL = :CDFILIAL
               AND NRLANCESTQ = :NRLANCESTQ
        ";
        $params = [
            "CDFILIAL" => $cdfilancestq,
            "NRLANCESTQ" => $nrlancestq
        ];
        $prepared = $this->connection->prepare($query);
        $prepared->execute($params);
    }
    //Consulta os produtos da requisição por serviço
    private function consultaProdutosDaRequisicao($cdfilial, $cdservfili, $dtcardapio, $nrrequextrre) {
        $query = "
            SELECT P.CDPRODUTO, P.CDTIPOPRAT, P.CDFILAUXPRAT, P.CDPRATO, P.IDPREVTPPRAT,
            SUM(P.QTPREVRETI) AS QTPREVRETI,
            SUM(NVL(P.QTPREVEXTRA, 0)) QTPREVEXTRA,
            SUM(NVL(P.QTPREVDEVOL, 0)) QTPREVDEVOL,
            SUM(NVL(P.QTPREVLIB, 0)) QTPREVLIB
            FROM PREVRETI P
            WHERE P.CDFILIAL = :CDFILIAL
            AND P.CDSERVFILI = :CDSERVFILI
            AND P.DTPREVRETI = :DTPREVRETI
            AND P.NRREQUEXTR = :NRREQUEXTRRE
            GROUP BY P.CDPRODUTO, P.CDTIPOPRAT, P.CDFILAUXPRAT, P.CDPRATO, P.IDPREVTPPRAT
        ";
        $prepared = $this->connection->prepare($query);
        $params = [
            "CDFILIAL" => $cdfilial,
            "CDSERVFILI" => $cdservfili,
            "DTPREVRETI" => $dtcardapio,
            "NRREQUEXTRRE" => $nrrequextrre
        ];
        $prepared->execute($params);
        return $prepared->fetchAll();
    }
    //Prepara objeto do produto da requisição para inserir
    private function preparaProdutoRequisicao($produto, $seq, $cdfilial, $nrretirada, $nrcardapio, $dtretirada, $cdfilancestq, $nrlancestq, $cdtpretipadr){
        $nrSeq = str_pad(strval($seq), 4, "0", STR_PAD_LEFT);
        $qtPrevista = $produto["QTPREVRETI"];
        $qtExtra = $produto['QTPREVEXTRA'];
        $qtDevolvida = $produto['QTPREVDEVOL'];
        $qtRetirada = $produto['QTPREVLIB'];
        $qtsaldoretirada = 0;
        if($qtExtra > 0 || $qtDevolvida > 0 || $qtRetirada > 0) {
            $qtRetirada = $qtPrevista + $qtExtra - $qtDevolvida;
        }
        if($qtRetirada < 0) {
            $qtExtra = 0;
            $qtDevolvida = 0;
            $qtRetirada = 0;
        } else {
            if ($qtRetirada > 0) {
                $qtsaldoretirada = $this->consultaSaldoDia($cdfilial, $dtretirada, $produto['CDPRODUTO'], ' ', ' ', ' ', ' ');
                if($qtRetirada > $qtsaldoretirada) {
                    $qtExtra = 0;
                    $qtDevolvida = 0;
                    $qtRetirada = 0;    
                }
            }
        }
        
        return [
            'CDPRODUTO' => $produto['CDPRODUTO'],
            'CDFILIAL' => $cdfilial,
            'NRCARDAPIO' => $nrcardapio,
            'NRRETIRADA' => $nrretirada,
            'NRLANCESTQ' => $nrlancestq,
            'DTRETIRADA' => $dtretirada,
            'CDFILAUXPRAT' => $produto['CDFILAUXPRAT'],
            'CDPRATO' => $produto['CDPRATO'],
            'CDTIPOPRAT' => $produto['CDTIPOPRAT'],
            'CDALMOXARIFE' => ' ',
            'CDLOCALESTOQ' => ' ',
            'CDFILANCESTQ' => $cdfilancestq,
            'NRLOTEESTQ' => ' ',
            'NRSUBLOTE' => ' ',
            'NRSEQUITRETI' => $nrSeq, 
            'NRSEQUITEM' => $nrSeq,
            'QTPREVISRETI' => $qtPrevista,
            'QTPREVISRETI_ANTERIOR' => 0,
            'QTEXTRARETI' => $qtExtra,
            'QTEXTRARETI_ANTERIOR' => 0,
            'QTDEVOLVRETI' => $qtDevolvida,
            'QTDEVOLVRETI_ANTERIOR' => 0,
            'QTRETIRADA' => $qtRetirada,
            'QTRETIRADA_ANTERIOR' => 0,
            'CDTPRETIPADR' => $cdtpretipadr,
            'QTSALDORETIRADA' => $qtsaldoretirada,
            'QTSALDORETIRADA_ANTERIOR' => 0,
            'QTLANCTOEST_ANTERIOR' => 0,
            'QTTOTLANCTO_ANTERIOR' => 0,
            'VRUNILANCTO_ANTERIOR' => 0,
            'VRTOTLANCTO_ANTERIOR' => 0,
            'IDEVOLESTOQ' => 'N',
            'IDEVOLESTOQ_ANTERIOR' => 'N',
            'INDICE' => -1,
            'PRODUTO_NOVO' => true,
            'EXCLUIR' => false
        ];
    }
    //Insere os produtos da requisição existente na nova retirada
    private function insereProdutosDaRequisicao($cdfilial, $cdservfili, $dtcardapio, $nrretirada, $nrcardapio, $dtretirada, $nrrequextrre, $cdtpretipadr, $cdfilancestq, $nrlancestq) {
        $produtosRequisicao = $this->consultaProdutosDaRequisicao($cdfilial, $cdservfili, $dtcardapio, $nrrequextrre);
        $seq = 0;
        foreach($produtosRequisicao as $prod) {
            $produto_preparado = $this->preparaProdutoRequisicao($prod, $seq, $cdfilial, $nrretirada, $nrcardapio, $dtretirada, $cdfilancestq, $nrlancestq, $cdtpretipadr);
            if($produto_preparado) {
                $this->insereItemReti($produto_preparado);
                $seq++;
            }
        }
    }
    //Insere item da retirada
    private function insereItemReti($produto){
        //Insere o item de estoque
        $this->insereItemEstoque($produto);
        
        $query = "
            INSERT INTO ITEMRETI
            (CDFILIAL, NRRETIRADA, NRCARDAPIO, CDPRODUTO,
             QTPREVISRETI, QTEXTRARETI, QTDEVOLVRETI, QTRETIRADA,
             NRSEQUITRETI, CDFIITLANEST, NRLANCESTQ, NRSEQUITEM,
             CDTIPORETI, CDTIPOPRAT, CDPRATO, CDFILAUXPRAT, QTSALDORETIRADA,
             NRORG, NRORGULTATU, NRORGINCLUSAO,
             DTULTATU, DTINCLUSAO,
             CDOPERULTATU, CDOPERINCLUSAO)
            VALUES 
            (:CDFILIAL, :NRRETIRADA, :NRCARDAPIO, :CDPRODUTO,
             :QTPREVISRETI, :QTEXTRARETI, :QTDEVOLVRETI, :QTRETIRADA,
             :NRSEQUITRETI, :CDFIITLANEST, :NRLANCESTQ, :NRSEQUITEM,
             :CDTIPORETI, :CDTIPOPRAT, :CDPRATO, :CDFILAUXPRAT, :QTSALDORETIRADA,
             :NRORG, :NRORG, :NRORG,
             SYSDATE, SYSDATE,
             :CDOPERADORLOGADO, :CDOPERADORLOGADO)
        ";
        $params = [
            "CDFILIAL" => $produto["CDFILIAL"],
            "NRRETIRADA" => $produto["NRRETIRADA"],
            "NRCARDAPIO" => $produto["NRCARDAPIO"],
            "CDPRODUTO" => $produto["CDPRODUTO"],
            "QTPREVISRETI" => $produto["QTPREVISRETI"],
            "QTEXTRARETI" => $produto['QTEXTRARETI'],
            "QTDEVOLVRETI" => $produto['QTDEVOLVRETI'],
            "QTRETIRADA" => $produto['QTRETIRADA'],
            "NRSEQUITRETI" => $produto['NRSEQUITRETI'],
            "CDFIITLANEST" => $produto["CDFILANCESTQ"],
            "NRLANCESTQ" => $produto["NRLANCESTQ"],
            "NRSEQUITEM" => $produto['NRSEQUITRETI'],
            "CDTIPORETI" => $produto["CDTPRETIPADR"],
            "CDTIPOPRAT" => $produto["CDTIPOPRAT"],
            "CDPRATO" => $produto["CDPRATO"],
            "CDFILAUXPRAT" => $produto["CDFILAUXPRAT"],
            "QTSALDORETIRADA" => $produto["QTSALDORETIRADA"],
            "NRORG" => $this->environment->getNrOrg(),
            "CDOPERADORLOGADO" => $this->environment->getCdOperador()
        ];
        $prepared = $this->connection->prepare($query);
        $prepared->execute($params);    
    }
    //Atualiza item da retirada
    private function atualizaItemReti($produto, $restaurando = false){
        $this->atualizaItemEstoque($produto, $restaurando);
        
        $qtprevisreti = $produto['QTPREVISRETI_ANTERIOR'];
        $qtextrareti = $produto['QTEXTRARETI_ANTERIOR'];
        $qtdevolvreti = $produto['QTDEVOLVRETI_ANTERIOR'];
        $qtretirada = $produto['QTRETIRADA_ANTERIOR'];
        $qtsaldoretirada = $produto['QTSALDORETIRADA_ANTERIOR'];
        $idevolestoq = $produto['IDEVOLESTOQ_ANTERIOR'];
        if(!$restaurando){
            $qtprevisreti = $produto['QTPREVISRETI'];
            $qtextrareti = $produto['QTEXTRARETI'];
            $qtdevolvreti = $produto['QTDEVOLVRETI'];
            $qtretirada = $produto['QTRETIRADA'];
            $qtsaldoretirada = $produto['QTSALDORETIRADA'];
            $idevolestoq = $produto['IDEVOLESTOQ'];
        }
        
        $query = "
            UPDATE ITEMRETI
               SET QTPREVISRETI = :QTPREVISRETI,
                   QTEXTRARETI = :QTEXTRARETI,
                   QTDEVOLVRETI = :QTDEVOLVRETI,
                   QTRETIRADA = :QTRETIRADA,
                   QTSALDORETIRADA = :QTSALDORETIRADA,
                   DTULTATU = SYSDATE,
                   CDOPERULTATU = :CDOPERADORLOGADO
             WHERE CDFILIAL = :CDFILIAL
               AND NRCARDAPIO = :NRCARDAPIO
               AND NRRETIRADA = :NRRETIRADA
               AND NRSEQUITRETI = :NRSEQUITRETI
        ";
        $params = [
            "QTPREVISRETI" => $produto["QTPREVISRETI"],
            "QTEXTRARETI" => $produto['QTEXTRARETI'],
            "QTDEVOLVRETI" => $produto['QTDEVOLVRETI'],
            "QTRETIRADA" => $produto['QTRETIRADA'],
            "QTSALDORETIRADA" => $produto["QTSALDORETIRADA"],
            "CDOPERADORLOGADO" => $this->environment->getCdOperador(),
            "CDFILIAL" => $produto["CDFILIAL"],
            "NRRETIRADA" => $produto["NRRETIRADA"],
            "NRCARDAPIO" => $produto["NRCARDAPIO"],
            "NRSEQUITRETI" => $produto['NRSEQUITRETI']
        ];
        $prepared = $this->connection->prepare($query);
        $prepared->execute($params);    
    }
    //Exclui item da retirada correspondente ao produto
    private function excluiItemReti($produto){
        $query = "
            DELETE FROM ITEMRETI
             WHERE CDFILIAL = :CDFILIAL
               AND NRCARDAPIO = :NRCARDAPIO
               AND NRRETIRADA = :NRRETIRADA
               AND NRSEQUITRETI = :NRSEQUITRETI
        ";
        $params = [
            "CDFILIAL" => $produto['CDFILIAL'],
            "NRCARDAPIO" => $produto['NRCARDAPIO'],
            "NRRETIRADA" => $produto['NRRETIRADA'],
            "NRSEQUITRETI" => $produto['NRSEQUITRETI']
        ];
        $prepared = $this->connection->prepare($query);
        $prepared->execute($params);
        
        $this->excluiItemEstoque($produto);
    }
    //Exclui todos os itens da retirada
    private function excluiTodosItensReti($cdfilial, $nrcardapio, $nrretirada, $cdfilancestq, $nrlancestq){
        $query = "
            DELETE FROM ITEMRETI
             WHERE CDFILIAL = :CDFILIAL
               AND NRCARDAPIO = :NRCARDAPIO
               AND NRRETIRADA = :NRRETIRADA
        ";
        $params = [
            "CDFILIAL" => $cdfilial,
            "NRCARDAPIO" => $nrcardapio,
            "NRRETIRADA" => $nrretirada
        ];
        $prepared = $this->connection->prepare($query);
        $prepared->execute($params);
        
        $this->excluiTodosItensEstoque($cdfilancestq, $nrlancestq);
    }
    //Verifica se existem efetivos faturados no dia e serviço da retirada
    private function verificaEfetFatu($cdfilial, $cdservfili, $dtcardapio){
        $query = "
            SELECT S.CDFILIAL, S.CDSERVFILI, S.NMSERVICO
            FROM EFETCLIE E
            INNER JOIN SERVICO S ON S.CDFILIAL = E.CDFILIAL AND S.CDSERVFILI = E.CDSERVFILI
            WHERE E.CDFILIAL = :CDFILIAL
            AND E.CDSERVPLAN = :CDSERVFILI
            AND E.DTEFETCLIE = :DTEFETCLIE
            AND E.NRAPURACFAT IS NOT NULL
        ";
        $params = [
            "CDFILIAL" => $cdfilial,
            "CDSERVFILI" => $cdservfili,
            "DTEFETCLIE" => $dtcardapio
        ];
        $prepared = $this->connection->prepare($query);
        $prepared->execute($params);
        return $prepared->fetchAll();
    }
    //Desfaz as alterações realizadas no produto, se for produto novo, exclui
    private function desfazAlteracoesProduto($produto){
        if($produto['PRODUTO_NOVO']){
            $this->excluiItemReti($produto);
        } else {
            $this->atualizaItemReti($produto, true);
        }
    }
    //Aplica rotina de estoque, retorna false caso haja irregularidades;
    private function atualizaEstoque($cdfilial, $nrlancestq, &$produtos_aplicados, $nrcardapio, $nrretirada, $cdfilancestq){
        $estoqueValido = true;
        try{
            $this->ValidacoesEstoqueService->atualizaEstoque($cdfilancestq, $nrlancestq, false, true, $this->abortEstqAtIrregularities);    
        }catch(\Exception $e){
            if (!strpos($e->getMessage(), 'alguns itens se tornasse irregular')) {
                throw $e;
            }
            $estoqueValido = false;
            //Se alguma transação ainda estiver ativa e está marcado para rollback, elimina as transações ainda pendentes
            while($this->connection->isTransactionActive() && $this->connection->isRollbackOnly())
                $this->connection->rollback();
        }
        if(!$estoqueValido){
            try{
                //É necessária nova transação visto que a rotina de atualização de estoque também realiza transação própria 
                $this->connection->beginTransaction();
                $produtosIrregulares = $this->consultaItensReti($cdfilial, $nrcardapio, $nrretirada, true);
                foreach($produtosIrregulares as $produto){
                    //Aponta irregularidades de estoque apenas para os produtos enviados
                    $produtoSemEstoque = array_filter($produtos_aplicados, function($prodAlt) use($produto) {
                        return ($prodAlt['NRSEQUITRETI'] == $produto['NRSEQUITRETI']);    
                    });

                    if(count($produtoSemEstoque) > 0){
                        $produtoSemEstoque = array_pop($produtoSemEstoque);
                        
                        // adicionado flag p/ verificar se exclui produto da retirada se sem estoque
                        if ($this->restauraRetiradaSeProdutoSemEstoque) {
                            $this->desfazAlteracoesProduto($produtoSemEstoque);    
                        }
                        $indice = $produtoSemEstoque['INDICE'];
                        $this->incluiMensagem("prodIrregEst", $this->cancelaEmErroDeProduto, $indice);
                    }
                }
                $produtosRetirada = $this->consultaItensReti($cdfilial, $nrcardapio, $nrretirada);
                if(count($produtosRetirada) == 0){
                    $this->removeRetirada($cdfilial, $nrcardapio, $nrretirada, $cdfilancestq, $nrlancestq);
                    $this->incluiMensagem("retiSemProdutos", true);
                }
                
                $this->connection->commit();
            }catch(\Exception $e){
                $this->connection->rollback();
                throw $e;
            }
        }
    }
    //Realiza a retirada do serviço e produtos
    private function realizaRetirada($cdfilial, $cdservfili, $dtretirada, $dtcardapio, $dslancestq, $produtos, $nrrequextrre=null, $nrcard=null, $nrreti=null){
        try{
            $this->connection->beginTransaction();
            $cdtpretipadr = null;
            $paramplanejaValido = $this->validaParamPlaneja($cdfilial, $cdtpretipadr);
            if(!$paramplanejaValido) return false;
            $retirada = null;
            if(!empty($nrcard) && !empty($nrreti)){
                $retirada = $this->consultaRetirada($cdfilial, $nrcard, null, $nrreti);
                if(!$retirada) {
                    $this->incluiMensagem('retiNExiste', true);
                } else {
                    $retirada['NOVA'] = false;
                }
            }
            else {
                if ($nrrequextrre == null) {
                    $nrrequextrre = '0000000000';
                }
                $retirada = $this->consultaOuInsereRetirada($cdfilial, $cdservfili, $dtretirada, $dtcardapio, $dslancestq, $nrrequextrre);
            }
            //Se teve algum problema na leitura ou criação da retirada
            if(!$this->Utils->semErros()){
                $this->connection->rollback();
                return false;
            }
            $nrcardapio = $retirada['NRCARDAPIO'];
            $nrretirada = $retirada['NRRETIRADA'];
            $cdfilancestq = $retirada['CDFILANCESTQ'];
            $nrlancestq = $retirada['NRLANCESTQ'];
            $dtretirada = $retirada['DTRETIRADA'];
            $dtcardapio = $retirada['DTCARDAPIO'];
            $cdservfili = $retirada['CDSERVFILI'];
            $this->retiradaPorPrato = ($retirada['IDRETIPRAT'] == 'S');
            
            //Insere os itens da previsão de consumo, caso existam
            if($this->geraDaRequisicao && $retirada['NOVA']) {
                $this->insereProdutosDaRequisicao($cdfilial, $cdservfili, $dtcardapio, $nrretirada,
                $nrcardapio, $dtretirada, $nrrequextrre, $cdtpretipadr,
                $cdfilancestq, $nrlancestq   );
            }
            
            $seq = $this->consultaProxNrItem($cdfilial, $nrcardapio, $nrretirada);
            
            /*Produtos que foram alterados/incluidos são armazenados aqui. 
            Ao evoluir o estoque, se algum deles apresentar erro, apaga/restaura linha dos mesmos*/
            $produtos_aplicados = array();
            $i = 0;
            $numProdutos = count($produtos);
            $semDuplicidade = true;
            /*Percorre cada um dos produtos enviados, e se forem válidos segue para a atualização do banco de dados*/
            for($i = 0; $i < $numProdutos; $i++){
                $produto = $this->validaERetornaProduto($produtos[$i], $i, $cdfilial, $nrcardapio, 
                                                        $nrretirada, $dtretirada, $nrlancestq, $cdtpretipadr, $cdfilancestq);
                if($produto){
                    if($produto['PRODUTO_NOVO']){
                        $produto['NRSEQUITRETI'] = str_pad(strval($seq), 4, "0", STR_PAD_LEFT);
                        $seq++;
                    }
                    if($this->registraProdutosAlteradosSemDuplicidade($produtos_aplicados, $produto)){
                        if($produto['EXCLUIR']) {
                            //Se o produto está sendo excluído da retirada, apaga o item e seu lançamento
                            $this->excluiItemReti($produto);
                        } else {
                            //Se novo produto cria novo sequencial a partir da numeração já existente na retirada
                            if($produto['PRODUTO_NOVO']){
                                $this->insereItemReti($produto);
                            } else{
                                $this->atualizaItemReti($produto);
                            }
                        }
                    }
                }
            }
            
            //Se nenhum dos produtos informados foi válido para registro na retirada
            if(count($produtos_aplicados) == 0 && $this->exigeProdutos){
                $this->incluiMensagem("semProdValido", true);
            }
            
            //Se chegou até o final com o status de SUCESO, aplica o commit, rollback caso contrário
            if($this->Utils->semErros()){
                $this->connection->commit();
            }
            else{
                $this->connection->rollback();
            }
        }catch(\Exception $e){
            $this->connection->rollback();
            throw $e;
        }
        //Aplica atualização se estoque caso a operação de retirada tenha sido realizada com sucesso
        if($this->Utils->semErros()){
            //Atualiza o estoque e verifica possíveis irregularidades
            $this->atualizaEstoque($cdfilial, $nrlancestq, $produtos_aplicados, $nrcardapio, $nrretirada, $cdfilancestq);
        }
        //Se retirada nova, concluída com sucesso, inclui a chave da nova retirada no retorno
        if($this->Utils->semErros() && $retirada['NOVA']){
            $this->incluiMensagem("retirada", false, [
                'CDFILIAL' => $retirada['CDFILIAL'],
                'NRCARDAPIO' => $retirada['NRCARDAPIO'],
                'NRRETIRADA' => $retirada['NRRETIRADA']
            ]);
        }
    }
    //Exclui retirada e todos os seus itens
    private function realizaExclusaoRetirada($cdfilial, $nrcardapio, $nrretirada){
        $retirada = $this->consultaRetirada($cdfilial, $nrcardapio, null, $nrretirada);
        if(!$retirada) {
            $this->incluiMensagem('retiNExiste', true);
            return false;
        }
        try {
            $this->connection->beginTransaction();
            $nrcardapio = $retirada['NRCARDAPIO'];
            $nrretirada = $retirada['NRRETIRADA'];
            $cdfilancestq = $retirada['CDFILANCESTQ'];
            $nrlancestq = $retirada['NRLANCESTQ'];
            $dtretirada = $retirada['DTRETIRADA'];
            $dtcardapio = $retirada['DTCARDAPIO'];
            $cdservfili = $retirada['CDSERVFILI'];
            
            $efetivosJaApurados = $this->verificaEfetFatu($cdfilial, $nrcardapio, $dtcardapio);
            if (count($efetivosJaApurados) > 0) {
                $this->incluiMensagem('efetJaFaturados', true, $efetivosJaApurados);
            }
            $this->excluiTodosItensReti($cdfilial, $nrcardapio, $nrretirada, $cdfilancestq, $nrlancestq);
            $this->ValidacoesEstoqueService->atualizaEstoque($cdfilancestq, $nrlancestq);
            $this->removeRetirada($cdfilial, $nrcardapio, $nrretirada, $cdfilancestq, $nrlancestq);
            //Se chegou até o final com o status de SUCESO, aplica o commit, rollback caso contrário
            if($this->Utils->semErros()){
                $this->connection->commit();
            }
            else{
                $this->connection->rollback();
            }
        } catch(\Exception $e){
            $this->connection->rollback();
            throw $e;
        }
    }
    /***
     * Inclui retirada para os dados informados
     * Parâmetro
     *  retirada: array associativo do tipo Retirada
     * Dados obrigatórios:
     *  CDFILIAL: Código da filial
     *  CDSERVFILI: Código do serviço
     *  DTRETIRADA: Data da retirada
     *  DTCARDAPIO: Data do cardápio
     *  DSLANCESTQ: Descrição do lançamento de estoque
     * Dados opcionais:
     *  NRREQUEXTRRE: Número da requisição lançada no sistema
     *  produtos: array associativo do tipo ProdutoRetirada (ver documentação)
     *  exigeProdutos: Indica se a lista de produtos a retirar é obrigatória
     *  cancelaEmErroDeProduto: Indica se caso um produto esteja inconsistente, cancela a operação dos demais
     *  geraDaRequisicao: Indica se inclui os produtos a partir da requisição
     *      
     * Retorno: Objeto de retorno padrão (ver README.md)
     * ***/
    public function incluirRetirada($retirada){
        try{
            $cdfilial = $this->Utils->valor($retirada, 'CDFILIAL', "");
            $cdservfili = $this->Utils->valor($retirada, 'CDSERVFILI', "");
            $dtretirada = $this->Utils->valor($retirada, 'DTRETIRADA', "");
            $dtcardapio = $this->Utils->valor($retirada, 'DTCARDAPIO', "");
            $dslancestq = $this->Utils->valor($retirada, 'DSLANCESTQ', "");
            $produtos = $this->Utils->valor($retirada, 'produtos');
            $nrrequextrre = $this->Utils->valor($retirada, 'NRREQUEXTRRE');
            $this->exigeProdutos = $this->Utils->valor($retirada, 'exigeProdutos', false);
            $this->cancelaEmErroDeProduto = $this->Utils->valor($retirada, 'cancelaEmErroDeProduto', true);
            $this->geraDaRequisicao = $this->Utils->valor($retirada, 'geraDaRequisicao', true);
            $this->restauraRetiradaSeProdutoSemEstoque = $this->Utils->valor($retirada, 'restauraRetSeProdSemEst', true);
            $this->abortEstqAtIrregularities = $this->Utils->valor($retirada, 'abortEstqAtIrregularities', true);
            
            $filialValida = $this->validaFilial($cdfilial);
            $descricaoValida = $this->validaDescricaoEstoque($dslancestq);
            $dtCardValida = $this->validaDtCard($dtcardapio);
            $dtRetiValida = $this->validaDtReti($dtretirada);
            
            if($filialValida && $descricaoValida && $dtCardValida && $dtRetiValida){
                $servicoValido = $this->validaServico($cdfilial, $cdservfili);
                
                $paramfilial = $this->consultaParamfilial($cdfilial);
                $periodoValido = $this->validaPeriodo($cdfilial, $dtretirada, $paramfilial);
                $produtosValido = true;
                if($this->exigeProdutos) {
                    $produtosValido = $this->validaListaProdutos($produtos);
                }
                if($servicoValido && $periodoValido && ( !$this->exigeProdutos || $produtosValido ) ){
                    $this->realizaRetirada($cdfilial, $cdservfili, $dtretirada, $dtcardapio,
                                          $dslancestq, $produtos, $nrrequextrre);
                }
            }
            
            return $this->Utils->retornar();
        }catch(\Exception $e){
            $this->Utils->incluiMensagem("erroNEsperado", true, $e->getMessage());
            return $this->Utils->retornar();
        }
    }
    /***
     * Realiza a inclusão ou alteração dos produtos da lista
     * Parâmetro
     *  retirada: array associativo do tipo Retirada
     * Dados obrigatórios:
     *  CDFILIAL: Código da filial
     *  NRCARDAPIO: Número do cardápio
     *  NRRETIRADA: Número da retirada
     * Dados opcionais
     *  cancelaEmErroDeProduto: Indica se caso um produto esteja inconsistente, cancela a operação dos demais
     * ***/
    public function atualizarRetirada($retirada){
        try{
            $cdfilial = $this->Utils->valor($retirada, 'CDFILIAL');
            $nrcardapio = $this->Utils->valor($retirada, 'NRCARDAPIO');
            $nrretirada = $this->Utils->valor($retirada, 'NRRETIRADA');
            $produtos = $this->Utils->valor($retirada, 'produtos');
            $this->exigeProdutos = true;
            $this->cancelaEmErroDeProduto = $this->Utils->valor($retirada, 'cancelaEmErroDeProduto', true);
            $produtosValido = $this->validaListaProdutos($produtos);
            if($produtosValido){
                $this->realizaRetirada($cdfilial, null, null, null,
                                      null, $produtos, null, $nrcardapio, $nrretirada);
            }
            
            return $this->Utils->retornar();
        } catch(\Exception $e){
            $this->Utils->incluiMensagem("erroNEsperado", true, $e->getMessage());
            return $this->Utils->retornar();
        }
    }
    /***
     * Realiza a exclusão completa da retirada
     * Parâmetro
     *  retirada: array associativo do tipo Retirada
     * Dados obrigatórios:
     *  CDFILIAL: Código da filial
     *  NRCARDAPIO: Número do cardápio
     *  NRRETIRADA: Número da retirada
     * ***/
    public function excluirRetirada($retirada){
        try{
            $cdfilial = $this->Utils->valor($retirada, 'CDFILIAL', "");
            $nrcardapio = $this->Utils->valor($retirada, 'NRCARDAPIO', "");
            $nrretirada = $this->Utils->valor($retirada, 'NRRETIRADA', "");
            $this->realizaExclusaoRetirada($cdfilial, $nrcardapio, $nrretirada);
            return $this->Utils->retornar();
        } catch(\Exception $e){
            $this->Utils->incluiMensagem("erroNEsperado", true, $e->getMessage());
            return $this->Utils->retornar();
        }
    }
}