# Tecfood Libraries Backend - v1

### Descrição

Ferramentas compartilhadas entre os módulos do tecfood. Centralização de regras de negócio críticas do sistema.


### **Requisitos para uso**

- **Composer.json**
    
    No arquivo composer.json, colocar a versão desejada para uso, exemplo:
    
    ```json
    "require": [
      /*Outras dependencias aqui*/
    	**"teknisa/tecfoodlib": ">=0.0.0"**
    ]
    ```
    
- **Parâmetro dcm_path**
    
    No arquivo xml onde os parâmetros do produto/módulo são definidos, deve-se configurar o dcm_path (criá-lo caso não exista). Note que o caminho é relativo ao index.php. Se existir template (arquivo_em_questao.xml.template), deve-se alterar diretamente no template.
    
    ```xml
    <?xml version="1.0"?>
    <container xmlns="http://symfony.com/schema/dic/services"
               xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
               xsi:schemaLocation="http://symfony.com/schema/dic/services http://symfony.com/schema/dic/services/services-1.0.xsd">
    
        <parameters>
            <!-- Outros parâmetros aqui -->
            <parameter key="dcm_path" type="collection">
    	          <!-- Outras configurações do dcm_path aqui -->
                **<parameter>../vendor/projetotecfood/entities/dcm</parameter>**
                **<parameter>../vendor/teknisa/entities/dcm</parameter>**
            </parameter>
        </parameters>
    </container> 
    ```
    
- **Inicialização**
    
    A instanciação e uso assume que o módulo/produto em questão, em seu bootstrap, já realiza as configurações necessárias do framework, teknisa libraries, Setforms2 no EntityManager.
    
    Acrescentar o seguinte código no bootstrap do módulo/produto, antes do registro do  serviço routeParser (/vendor/ deve ser substituído pelo caminho até o vendor de seu módulo/produto):
    
    ```php
    **require __DIR__ . "/vendor/teknisa/tecfoodLib/bootstrap.php";**
    /* Seu código até aqui */
    **bootstrapTecfoodLib($instanceManager, $routes, __DIR__ . "/vendor/");**
    /* Seu código até aqui*/
    $instanceManager->registerService("routeParser", new \Zeedhi\Framework\Routing\Parsers\JSONFormat($routes));
    ```
    
    No GruntFile do módulo, na rotina que cria o mapa de dataSources(geralmente é o backendStep), realizar as alterações abaixo (note a chamada da função **antes** da criação do arquivo dataSourceMapping):
    
    ```jsx
    module.exports = function(grunt) {
    **const bootstrapTecfoodLib = require('../backend/vendor/teknisa/tecfoodlib/bootstrap.js');**
    /* Seu código até aqui */
    **bootstrapTecfoodLib(grunt, srcFiles.datasources, '../backend/vendor/', './../vendor/');**
    grunt.file.write(dest.datasources, JSON.stringify(srcFiles.datasources, null, 2));
    ```
    
    A função bootstrapTecfoodLib recebe a variável grunt (criada no início de todo GruntFile), a variável onde o mapa de dataSources está sendo montado, o caminho **relativo ao gruntFile** para o diretório do vendor e o caminho **relativo ao index.php** para o diretório do vendor.
    
- **Retornos dos métodos da biblioteca**
    
    Todos os métodos da biblioteca retornam o objeto Retorno (array associativo).
    
    **Retorno**
    
    | campo | Descrição | Valores aceitos |
    | --- | --- | --- |
    | sucesso | Indica se a operação foi bem sucedida | boolean |
    | mensagens | lista de mensagens emitida pelo método, do tipo Mensagem | array |
    
    **Mensagem**
    
    | campo | Descrição | Valores aceitos |
    | --- | --- | --- |
    | msg | Código ou texto da mensagem | boolean |
    | complemento | Complemento | qualquer tipo, null caso não informado |

Leia a documentação completa: [Documentação](https://drive.teknisa.com/index.php/s/VX2JIsr0B2vHSK0)