<?php
namespace Tecfoodlib\Services;

class Utilities {
  protected $sucesso;
  protected $mensagens;
  public function __construct($is_dev_mode){
      $this->sucesso = true;
      $this->mensagens = array();
  }
  //Busca o valor de um array associativo, caso o parâmetro não exista aplica o valor default
  public function valor($array_assoc, $nome, $default = null){
      if(!$array_assoc || !is_array($array_assoc) || !is_string($nome)) return $default;
      if(!isset($array_assoc[$nome])) return $default;
      return $array_assoc[$nome];
  }
  
  //Registra uma mensagem para retorno padrão da biblioteca. Caso $erro seja true, troca o status de sucesso para false
  public function incluiMensagem($msg, $erro, $complemento=null){
      if($erro)
          $this->sucesso = false;
      if(!empty($msg) && is_string($msg)){
        $this->mensagens[] = [
          'msg' => $msg,
          'complemento' => $complemento
        ];
      }
  }
  //Indica o status atual baseado nas mensagens de erro registradas
  public function semErros(){
      return $this->sucesso;
  }
  
  //Retorna um array associativo em formato específico sucesso (boolean) e mensagens (caso existam)
  public function retornar(){
      
      return [
          'sucesso' => $this->sucesso,
          'mensagens' => $this->mensagens
      ];
  }
  
  //Verifica se o texto informado é uma data válida para o formato indicado
  public function dataValida($dataEmString, $formato='d/m/Y'){
      $dateTime = \DateTime::createFromFormat($formato, $dataEmString);
      
      if ($dateTime !== false && !array_sum($dateTime::getLastErrors())) {
          return true;
      } else {
          return false;
      }
  }
  //Retorna um datetime no formato especificado
  public function retornaDatetime($dataEmString, $formato='d/m/Y'){
      $dateTime = \DateTime::createFromFormat($formato, $dataEmString);

      if ($dateTime !== false && !array_sum($dateTime::getLastErrors())) {
          if($formato == 'd/m/Y'){
              $dateTime->setTime(0,0);
          }
          return $dateTime;
      } else {
          $dateTime = \DateTime::createFromFormat($formato.' H:i:s', $dataEmString);
          if ($dateTime !== false && !array_sum($dateTime::getLastErrors())) {
              return $dateTime;
          } else {
              return false;
          }
      }
  }
}